@@ -18,11 +18,14 @@
 #include <string>
 #include <sstream>
 
-#include "LIEF/visitors/Hash.hpp"
+#include "LIEF/MachO/hash.hpp"
 #include "LIEF/MachO/RelocationDyld.hpp"
 
 #include "pyMachO.hpp"
 
+namespace LIEF {
+namespace MachO {
+
 template<class T>
 using getter_t = T (RelocationDyld::*)(void) const;
 
@@ -30,7 +33,8 @@ template<class T>
 using setter_t = void (RelocationDyld::*)(T);
 
 
-void init_MachO_RelocationDyld_class(py::module& m) {
+template<>
+void create<RelocationDyld>(py::module& m) {
 
   py::class_<RelocationDyld, Relocation>(m, "RelocationDyld")
 
@@ -38,7 +42,7 @@ void init_MachO_RelocationDyld_class(py::module& m) {
     .def("__ne__", &RelocationDyld::operator!=)
     .def("__hash__",
         [] (const RelocationDyld& relocation) {
-          return LIEF::Hash::hash(relocation);
+          return Hash::hash(relocation);
         })
 
     .def("__str__",
@@ -51,3 +55,6 @@ void init_MachO_RelocationDyld_class(py::module& m) {
         });
 
 }
+
+}
+}
