@@ -13,7 +13,7 @@ import lief
 from lief import MachO
 
 from lief import Logger
-Logger.set_level(lief.LOGGING_LEVEL.INFO)
+Logger.set_level(lief.LOGGING_LEVEL.FATAL)
 
 terminal_rows, terminal_columns = 100, 100
 try:
@@ -23,6 +23,7 @@ except ValueError:
 
 terminal_columns = int(terminal_columns)
 terminal_rows    = int(terminal_rows)
+EXIT_STATUS = 0
 
 class exceptions_handler(object):
     func = None
@@ -38,6 +39,9 @@ class exceptions_handler(object):
         try:
             return self.func(*args, **kwargs)
         except self.exceptions as e:
+            global EXIT_STATUS
+            print("{} raised: {}".format(self.func.__name__, e))
+            EXIT_STATUS = 1
             if self.on_except_callback is not None:
                 self.on_except_callback(e)
             else:
@@ -98,15 +102,17 @@ def print_commands(binary):
 def print_libraries(binary):
 
     f_title = "|{:<30}|{:<10}|{:<16}|{:<22}|"
-    f_value = "|{:<30}|{:<10d}|{:<16x}|{:<22x}|"
+    f_value = "|{:<30}|{:<10d}|{:<16}|{:<22}|"
     print("== Libraries ==")
     print(f_title.format("Name", "Timestamp", "Current Version", "Compatibility Version"))
     for library in binary.libraries:
+        current_version_str = "{:d}.{:d}.{:d}".format(*library.current_version)
+        compatibility_version_str = "{:d}.{:d}.{:d}".format(*library.compatibility_version)
         print(f_value.format(
             library.name,
             library.timestamp,
-            library.current_version,
-            library.compatibility_version))
+            current_version_str,
+            compatibility_version_str))
     print("")
 
 @exceptions_handler(Exception)
@@ -225,6 +231,63 @@ def print_symbols(binary):
             libname))
     print("")
 
+@exceptions_handler(Exception)
+def print_symbol_command(binary):
+    print("== Symbol Command ==")
+
+    scmd = binary.symbol_command
+
+    format_str = "{:<17} {:<30}"
+    format_hex = "{:<17} 0x{:<28x}"
+    format_dec = "{:<17} {:<30d}"
+
+    print(format_hex.format("Symbol offset", scmd.symbol_offset))
+    print(format_dec.format("Number of symbols", scmd.numberof_symbols))
+
+    print(format_hex.format("String offset", scmd.strings_offset))
+    print(format_hex.format("String size", scmd.strings_size))
+
+    print("")
+
+@exceptions_handler(Exception)
+def print_dynamic_symbol_command(binary):
+    print("== Dynamic Symbol Command ==")
+
+    dyscmd = binary.dynamic_symbol_command
+
+    format_str = "{:<36} {:<30}"
+    format_hex = "{:<36} 0x{:<28x}"
+    format_dec = "{:<36} {:<30d}"
+
+    print(format_dec.format("First local symbol index", dyscmd.idx_local_symbol))
+    print(format_dec.format("Number of local symbols", dyscmd.nb_local_symbols))
+
+    print(format_dec.format("External symbol index", dyscmd.idx_external_define_symbol))
+    print(format_dec.format("Number of external symbols", dyscmd.nb_external_define_symbols))
+
+    print(format_dec.format("Undefined symbol index", dyscmd.idx_undefined_symbol))
+    print(format_dec.format("Number of undefined symbols", dyscmd.nb_undefined_symbols))
+
+    print(format_dec.format("Table of content offset", dyscmd.toc_offset))
+    print(format_dec.format("Number of entries in TOC", dyscmd.nb_toc))
+
+    print(format_hex.format("Module table offset", dyscmd.module_table_offset))
+    print(format_dec.format("Number of entries in module table", dyscmd.nb_module_table))
+
+    print(format_hex.format("External reference table offset", dyscmd.external_reference_symbol_offset))
+    print(format_dec.format("Number of external reference", dyscmd.nb_external_reference_symbols))
+
+    print(format_hex.format("Indirect symbols offset", dyscmd.indirect_symbol_offset))
+    print(format_dec.format("Number of indirect symbols", dyscmd.nb_indirect_symbols))
+
+    print(format_hex.format("External relocation offset", dyscmd.external_relocation_offset))
+    print(format_dec.format("Number of external relocations", dyscmd.nb_external_relocations))
+
+    print(format_hex.format("Local relocation offset", dyscmd.local_relocation_offset))
+    print(format_dec.format("Number of local relocations", dyscmd.nb_local_relocations))
+
+    print("")
+
 
 @exceptions_handler(Exception)
 def print_uuid(binary):
@@ -268,6 +331,21 @@ def print_thread_command(binary):
 
     print("")
 
+@exceptions_handler(Exception)
+def print_rpath_command(binary):
+
+    format_str = "{:<13} {:<30}"
+    format_hex = "{:<13} 0x{:<28x}"
+    format_dec = "{:<13} {:<30d}"
+
+    print("== Rpath Command ==")
+    cmd = binary.rpath
+    print("Path: {}".format(cmd.path))
+
+
+    print("")
+
+
 
 @exceptions_handler(Exception)
 def print_dylinker(binary):
@@ -294,6 +372,66 @@ def print_function_starts(binary):
 
     print("")
 
+@exceptions_handler(Exception)
+def print_data_in_code(binary):
+    format_str = "{:<13} {:<30}"
+    format_hex = "{:<13} 0x{:<28x}"
+    format_dec = "{:<13} {:<30d}"
+
+    print("== Data In Code ==")
+
+    datacode = binary.data_in_code
+
+    print(format_hex.format("Offset:", datacode.data_offset))
+    print(format_hex.format("Size:",   datacode.data_size))
+    print("")
+    for entry in datacode.entries:
+        type_str = str(entry.type).split(".")[-1]
+        print("- {:<14}: 0x{:x} ({:d} bytes)".format(type_str, entry.offset, entry.length))
+    print("")
+
+
+@exceptions_handler(Exception)
+def print_segment_split_info(binary):
+    format_str = "{:<13} {:<30}"
+    format_hex = "{:<13} 0x{:<28x}"
+    format_dec = "{:<13} {:<30d}"
+
+    print("== Segment Split Info ==")
+
+    sinfo = binary.segment_split_info
+
+    print(format_hex.format("Offset:", sinfo.data_offset))
+    print(format_hex.format("Size:",   sinfo.data_size))
+
+    print("")
+
+
+@exceptions_handler(Exception)
+def print_sub_framework(binary):
+    format_str = "{:<13} {:<30}"
+    format_hex = "{:<13} 0x{:<28x}"
+    format_dec = "{:<13} {:<30d}"
+
+    print("== Sub Framework ==")
+
+    sinfo = binary.sub_framework
+    print(format_str.format("Umbrella:", sinfo.umbrella))
+
+    print("")
+
+@exceptions_handler(Exception)
+def print_dyld_environment(binary):
+    format_str = "{:<13} {:<30}"
+    format_hex = "{:<13} 0x{:<28x}"
+    format_dec = "{:<13} {:<30d}"
+
+    print("== Dyld Environment ==")
+
+    env = binary.dyld_environment
+    print(format_str.format("Value:", env.value))
+
+    print("")
 
 
 @exceptions_handler(Exception)
@@ -348,6 +486,47 @@ def print_dyld_info(binary):
 
     print("")
 
+
+@exceptions_handler(Exception)
+def print_rebase_opcodes(binary):
+    print("== Rebase opcodes ==")
+
+    print(binary.dyld_info.show_rebases_opcodes)
+
+    print("")
+
+@exceptions_handler(Exception)
+def print_bind_opcodes(binary):
+    print("== Bind opcodes ==")
+
+    print(binary.dyld_info.show_bind_opcodes)
+
+    print("")
+
+@exceptions_handler(Exception)
+def print_weak_bind_opcodes(binary):
+    print("== Weak bind opcodes ==")
+
+    print(binary.dyld_info.show_weak_bind_opcodes)
+
+    print("")
+
+@exceptions_handler(Exception)
+def print_lazy_bind_opcodes(binary):
+    print("== Lazy bind opcodes ==")
+
+    print(binary.dyld_info.show_lazy_bind_opcodes)
+
+    print("")
+
+@exceptions_handler(Exception)
+def print_export_trie(binary):
+    print("== Export trie ==")
+
+    print(binary.dyld_info.show_export_trie)
+
+    print("")
+
 @exceptions_handler(Exception)
 def print_source_version(binary):
     print("== Source Version ==")
@@ -436,6 +615,23 @@ def print_relocations(binary):
 
     print("")
 
+@exceptions_handler(Exception)
+def print_encryption_info(binary):
+
+    format_str = "{:<13} {:<30}"
+    format_hex = "{:<13} 0x{:<28x}"
+    format_dec = "{:<13} {:<30d}"
+
+    print("== Encryption Info ==")
+    cmd = binary.encryption_info
+
+    print(format_hex.format("Offset:", cmd.crypt_offset))
+    print(format_hex.format("Size:",   cmd.crypt_size))
+    print(format_dec.format("ID:",     cmd.crypt_id))
+
+    print("")
+
+
 
 def main():
     parser = argparse.ArgumentParser(usage='%(prog)s [options] <macho-file>')
@@ -491,6 +687,10 @@ def main():
             action='store_true', dest='show_function_starts',
             help='Display the FunctionStarts command')
 
+    parser.add_argument('--rebase-opcodes',
+            action='store_true', dest='show_rebase_opcodes',
+            help='Display the "Rebase" opcodes')
+
     parser.add_argument('--source-version',
             action='store_true', dest='show_source_version',
             help="Display the 'Source Version' command")
@@ -503,6 +703,58 @@ def main():
             action='store_true', dest='show_thread_command',
             help="Display the 'Thread Command' command")
 
+    parser.add_argument('--rpath-command',
+            action='store_true', dest='show_rpath_command',
+            help="Display the 'Rpath Command' command")
+
+    parser.add_argument('--symbol-command',
+            action='store_true', dest='show_symbol_command',
+            help="Display the 'Symbol Command' command")
+
+    parser.add_argument('--dynamic-symbol-command',
+            action='store_true', dest='show_dynamic_symbol_command',
+            help="Display the 'Symbol Command' command")
+
+    parser.add_argument('--data-in-code',
+            action='store_true', dest='show_data_in_code',
+            help="Display the 'Data In Code' command")
+
+    parser.add_argument('--segment-split-info',
+            action='store_true', dest='show_segment_split_info',
+            help="Display the 'Segment Split Info' command")
+
+    parser.add_argument('--sub-framework',
+            action='store_true', dest='show_sub_framework',
+            help="Display the 'Sub Framework' command")
+
+    parser.add_argument('--dyld-environment',
+            action='store_true', dest='show_dyld_env',
+            help="Display the 'Dyld Environment' command")
+
+    parser.add_argument('--encryption-info',
+            action='store_true', dest='show_encrypt_info',
+            help="Display the 'Encryption Info' command")
+
+    parser.add_argument('--bind-opcodes',
+            action='store_true', dest='show_bind_opcodes',
+            help='Display the "Bind" opcodes')
+
+    parser.add_argument('--weak-bind-opcodes',
+            action='store_true', dest='show_weak_bind_opcodes',
+            help='Display the "Weak Bind" opcodes')
+
+    parser.add_argument('--lazy-bind-opcodes',
+            action='store_true', dest='show_lazy_bind_opcodes',
+            help='Display the "Lazy Bind" opcodes')
+
+    parser.add_argument('--export-trie',
+            action='store_true', dest='show_export_trie',
+            help='Display the export trie')
+
+    parser.add_argument('--opcodes',
+            action='store_true', dest='show_opcodes',
+            help='Display the bind and rebase opcodes')
+
     parser.add_argument("binary",
             metavar="<macho-file>",
             help='Target Mach-O File')
@@ -565,6 +817,50 @@ def main():
         if (args.show_thread_command or args.show_all) and binary.has_thread_command:
             print_thread_command(binary)
 
+        if (args.show_rpath_command or args.show_all) and binary.has_rpath:
+            print_rpath_command(binary)
+
+        if (args.show_symbol_command or args.show_all) and binary.has_symbol_command:
+            print_symbol_command(binary)
+
+        if (args.show_dynamic_symbol_command or args.show_all) and binary.has_dynamic_symbol_command:
+            print_dynamic_symbol_command(binary)
+
+        if (args.show_data_in_code or args.show_all) and binary.has_data_in_code:
+            print_data_in_code(binary)
+
+        if (args.show_segment_split_info or args.show_all) and binary.has_segment_split_info:
+            print_segment_split_info(binary)
+
+        if (args.show_sub_framework or args.show_all) and binary.has_sub_framework:
+            print_sub_framework(binary)
+
+        if (args.show_dyld_env or args.show_all) and binary.has_dyld_environment:
+            print_dyld_environment(binary)
+
+        if (args.show_encrypt_info or args.show_all) and binary.has_encryption_info:
+            print_encryption_info(binary)
+
+        if (args.show_rpath_command or args.show_all) and binary.has_rpath:
+            print_rpath_command(binary)
+
+        if (args.show_rebase_opcodes or args.show_opcodes) and binary.has_dyld_info:
+            print_rebase_opcodes(binary)
+
+        if (args.show_bind_opcodes  or args.show_opcodes) and binary.has_dyld_info:
+            print_bind_opcodes(binary)
+
+        if (args.show_weak_bind_opcodes or args.show_opcodes) and binary.has_dyld_info:
+            print_weak_bind_opcodes(binary)
+
+        if (args.show_lazy_bind_opcodes or args.show_opcodes) and binary.has_dyld_info:
+            print_lazy_bind_opcodes(binary)
+
+        if (args.show_export_trie or args.show_opcodes) and binary.has_dyld_info:
+            print_export_trie(binary)
+
+        sys.exit(EXIT_STATUS)
+
 
 if __name__ == "__main__":
     main()
