@@ -9,7 +9,7 @@ from lief import PE
 from lief.PE import oid_to_string
 
 from lief import Logger
-Logger.set_level(lief.LOGGING_LEVEL.INFO)
+Logger.set_level(lief.LOGGING_LEVEL.FATAL)
 
 from optparse import OptionParser
 import sys
@@ -273,6 +273,18 @@ def print_debug(binary):
     print(format_hex.format("Size of data:",        debug.sizeof_data))
     print(format_hex.format("Address of raw data:", debug.addressof_rawdata))
     print(format_hex.format("Pointer to raw data:", debug.pointerto_rawdata))
+
+    if debug.has_code_view:
+        code_view = debug.code_view
+        cv_signature = code_view.cv_signature
+
+        if cv_signature in (lief.PE.CODE_VIEW_SIGNATURES.PDB_70, lief.PE.CODE_VIEW_SIGNATURES.PDB_70):
+            sig_str = " ".join(map(lambda e : "{:02x}".format(e), code_view.signature))
+            print(format_str.format("Code View Signature:", str(cv_signature).split(".")[-1]))
+            print(format_str.format("Signature:", sig_str))
+            print(format_dec.format("Age:", code_view.age))
+            print(format_str.format("Filename:", code_view.filename))
+
     print("")
 
 
