@@ -54,15 +54,15 @@ class Parser;
 class Builder;
 
 //! @brief Class which represent an ELF binary
-class DLL_PUBLIC Binary : public LIEF::Binary {
+class LIEF_API Binary : public LIEF::Binary {
   friend class Parser;
   friend class Builder;
 
   public:
     Binary(const std::string& name, ELF_CLASS type);
 
-    Binary& operator=(const Binary& copy) = delete;
-    Binary(const Binary& copy)            = delete;
+    Binary& operator=(const Binary& ) = delete;
+    Binary(const Binary& copy) = delete;
 
     //! @brief Return binary's class (ELF32 or ELF64)
     ELF_CLASS type(void) const;
@@ -228,6 +228,28 @@ class DLL_PUBLIC Binary : public LIEF::Binary {
     it_symbols       symbols(void);
     it_const_symbols symbols(void) const;
 
+    //! Export the given symbol and create it if it doesn't exist
+    Symbol& export_symbol(const Symbol& symbol);
+
+    //! Export the symbol with the given name and create it if it doesn't exist
+    Symbol& export_symbol(const std::string& symbol_name, uint64_t value = 0);
+
+    //! Check if the symbol with the given ``name`` exists in the dynamic symbol table
+    bool has_dynamic_symbol(const std::string& name) const;
+
+    //! Get the dynamic symbol from the given name
+    const Symbol& get_dynamic_symbol(const std::string& name) const;
+
+    Symbol& get_dynamic_symbol(const std::string& name);
+
+    //! Check if the symbol with the given ``name`` exists in the static symbol table
+    bool has_static_symbol(const std::string& name) const;
+
+    //! Get the static symbol from the given name
+    const Symbol& get_static_symbol(const std::string& name) const;
+
+    Symbol& get_static_symbol(const std::string& name);
+
     //! @brief Remove symbols with the given name in boths
     //!   * dynamic symbols
     //!   * static symbols
@@ -270,8 +292,11 @@ class DLL_PUBLIC Binary : public LIEF::Binary {
     //! @brief Add a static symbol
     Symbol& add_static_symbol(const Symbol& symbol);
 
-    //! @brief Add a dynamic symbol
-    Symbol& add_dynamic_symbol(const Symbol& symbol);
+    //! @brief Add a dynamic symbol with the associated SymbolVersion
+    Symbol& add_dynamic_symbol(const Symbol& symbol, const SymbolVersion& version = SymbolVersion::global());
+
+    //! Create a symbol for the function at the given address and export it
+    Symbol& add_exported_function(uint64_t address, const std::string& name = "");
 
     //! @brief Add a library as dependency
     DynamicEntryLibrary& add_library(const std::string& library_name);
@@ -305,7 +330,7 @@ class DLL_PUBLIC Binary : public LIEF::Binary {
     //!
     //! @param[in] address Address to patch
     //! @param[in] patch_value Patch to apply
-    virtual void patch_address(uint64_t address, const std::vector<uint8_t>& patch_value) override;
+    virtual void patch_address(uint64_t address, const std::vector<uint8_t>& patch_value, LIEF::Binary::VA_TYPES addr_type = LIEF::Binary::VA_TYPES::AUTO) override;
 
 
     //! @brief Patch the address with the given value
@@ -313,7 +338,7 @@ class DLL_PUBLIC Binary : public LIEF::Binary {
     //! @param[in] address Address to patch
     //! @param[in] patch_value Patch to apply
     //! @param[in] size Size of the value in **bytes** (1, 2, ... 8)
-    virtual void patch_address(uint64_t address, uint64_t patch_value, size_t size = sizeof(uint64_t)) override;
+    virtual void patch_address(uint64_t address, uint64_t patch_value, size_t size = sizeof(uint64_t), LIEF::Binary::VA_TYPES addr_type = LIEF::Binary::VA_TYPES::AUTO) override;
 
     //! @brief Patch the imported symbol with the ``address``
     //!
@@ -389,6 +414,10 @@ class DLL_PUBLIC Binary : public LIEF::Binary {
     const Note& get(NOTE_TYPES type) const;
     Note&       get(NOTE_TYPES type);
 
+    //! @brief Return the **first** ELF::Section associated with the given type
+    const Section& get(ELF_SECTION_TYPES type) const;
+    Section&       get(ELF_SECTION_TYPES type);
+
     //! @brief Check if an ELF::DynamicEntry associated with the given tag
     //! exists.
     bool has(DYNAMIC_TAGS tag) const;
@@ -401,8 +430,13 @@ class DLL_PUBLIC Binary : public LIEF::Binary {
     //! exists.
     bool has(NOTE_TYPES type) const;
 
+    //! @brief Check if a ELF::Section associated with the given type
+    //! exists.
+    bool has(ELF_SECTION_TYPES type) const;
+
     //! @brief Return the content located at virtual address
-    virtual std::vector<uint8_t> get_content_from_virtual_address(uint64_t virtual_address, uint64_t size) const override;
+    virtual std::vector<uint8_t> get_content_from_virtual_address(uint64_t virtual_address, uint64_t size,
+        LIEF::Binary::VA_TYPES addr_type = LIEF::Binary::VA_TYPES::AUTO) const override;
 
     //! @brief Method so that the ``visitor`` can visit us
     virtual void accept(LIEF::Visitor& visitor) const override;
@@ -429,6 +463,9 @@ class DLL_PUBLIC Binary : public LIEF::Binary {
 
     virtual std::ostream& print(std::ostream& os) const override;
 
+    bool operator==(const Binary& rhs) const;
+    bool operator!=(const Binary& rhs) const;
+
 
     Binary& operator+=(const DynamicEntry& entry);
     Binary& operator+=(const Section& section);
@@ -450,7 +487,10 @@ class DLL_PUBLIC Binary : public LIEF::Binary {
     Note&       operator[](NOTE_TYPES type);
     const Note& operator[](NOTE_TYPES type) const;
 
-  private:
+    Section&       operator[](ELF_SECTION_TYPES type);
+    const Section& operator[](ELF_SECTION_TYPES type) const;
+
+  protected:
     Binary(void);
 
     //! @brief Return an abstraction of binary's section: LIEF::Section
@@ -464,7 +504,7 @@ class DLL_PUBLIC Binary : public LIEF::Binary {
     virtual LIEF::symbols_t          get_abstract_symbols(void) override;
     virtual LIEF::relocations_t      get_abstract_relocations(void) override;
 
-    template<int ARCH>
+    template<ELF::ARCH ARCH>
     void patch_relocations(uint64_t from, uint64_t shift);
 
     template<class T>
@@ -476,14 +516,15 @@ class DLL_PUBLIC Binary : public LIEF::Binary {
     void shift_symbols(uint64_t from, uint64_t shift);
     void shift_relocations(uint64_t from, uint64_t shift);
 
-    template<int OBJECT_TYPE, bool note = false>
+    template<E_TYPE OBJECT_TYPE, bool note = false>
     Segment& add_segment(const Segment& segment, uint64_t base);
 
-    template<int PT>
+    template<SEGMENT_TYPES PT>
     Segment& extend_segment(const Segment& segment, uint64_t size);
 
     template<bool LOADED>
     Section& add_section(const Section& section);
+    symbols_t static_dyn_symbols(void) const;
 
     //! The binary type
     //! (i.e. `ELF32` or `ELF64`)
