@@ -63,6 +63,11 @@ static const std::map<RELOC_x86_64, uint32_t> relocation_x86_64_sizes {
   {RELOC_x86_64::R_X86_64_TLSDESC_CALL,     0 },
   {RELOC_x86_64::R_X86_64_TLSDESC,         64 },
   {RELOC_x86_64::R_X86_64_IRELATIVE,       64 },
+  {RELOC_x86_64::R_X86_64_RELATIVE64,      64 },
+  {RELOC_x86_64::R_X86_64_PC32_BND,        32 },
+  {RELOC_x86_64::R_X86_64_PLT32_BND,       32 },
+  {RELOC_x86_64::R_X86_64_GOTPCRELX,       32 },
+  {RELOC_x86_64::R_X86_64_REX_GOTPCRELX,   32 },
 };
 
 static const std::map<RELOC_i386, uint32_t> relocation_i386_sizes {
@@ -376,6 +381,153 @@ static const std::map<RELOC_AARCH64, uint32_t> relocation_AARCH64_sizes {
   { RELOC_AARCH64::R_AARCH64_IRELATIVE,                    64 },
 };
 
+static const std::map<RELOC_POWERPC32, uint32_t> relocation_PPC_sizes {
+  { RELOC_POWERPC32::R_PPC_NONE,              0 },
+  { RELOC_POWERPC32::R_PPC_ADDR32,            32 },
+  { RELOC_POWERPC32::R_PPC_ADDR24,            32 },
+  { RELOC_POWERPC32::R_PPC_ADDR16,            16 },
+  { RELOC_POWERPC32::R_PPC_ADDR16_LO,         16 },
+  { RELOC_POWERPC32::R_PPC_ADDR16_HI,         16 },
+  { RELOC_POWERPC32::R_PPC_ADDR16_HA,         16 },
+  { RELOC_POWERPC32::R_PPC_ADDR14,            32 },
+  { RELOC_POWERPC32::R_PPC_ADDR14_BRTAKEN,    32 },
+  { RELOC_POWERPC32::R_PPC_ADDR14_BRNTAKEN,   32 },
+  { RELOC_POWERPC32::R_PPC_REL24,             32 },
+  { RELOC_POWERPC32::R_PPC_REL14,             32 },
+  { RELOC_POWERPC32::R_PPC_REL14_BRTAKEN,     32 },
+  { RELOC_POWERPC32::R_PPC_REL14_BRNTAKEN,    32 },
+  { RELOC_POWERPC32::R_PPC_GOT16,             16 },
+  { RELOC_POWERPC32::R_PPC_GOT16_LO,          16 },
+  { RELOC_POWERPC32::R_PPC_GOT16_HI,          16 },
+  { RELOC_POWERPC32::R_PPC_GOT16_HA,          16 },
+  { RELOC_POWERPC32::R_PPC_PLTREL24,          32 },
+  { RELOC_POWERPC32::R_PPC_JMP_SLOT,          0 },
+  { RELOC_POWERPC32::R_PPC_RELATIVE,          32 },
+  { RELOC_POWERPC32::R_PPC_LOCAL24PC,         32 },
+  { RELOC_POWERPC32::R_PPC_REL32,             32 },
+  { RELOC_POWERPC32::R_PPC_TLS,               32 },
+  { RELOC_POWERPC32::R_PPC_DTPMOD32,          32 },
+  { RELOC_POWERPC32::R_PPC_TPREL16,           16 },
+  { RELOC_POWERPC32::R_PPC_TPREL16_LO,        16 },
+  { RELOC_POWERPC32::R_PPC_TPREL16_HI,        16 },
+  { RELOC_POWERPC32::R_PPC_TPREL16_HA,        16 },
+  { RELOC_POWERPC32::R_PPC_TPREL32,           32 },
+  { RELOC_POWERPC32::R_PPC_DTPREL16,          16 },
+  { RELOC_POWERPC32::R_PPC_DTPREL16_LO,       16 },
+  { RELOC_POWERPC32::R_PPC_DTPREL16_HI,       16 },
+  { RELOC_POWERPC32::R_PPC_DTPREL16_HA,       16 },
+  { RELOC_POWERPC32::R_PPC_DTPREL32,          32 },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSGD16,       16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSGD16_LO,    16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSGD16_HI,    16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSGD16_HA,    16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSLD16,       16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSLD16_LO,    16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSLD16_HI,    16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSLD16_HA,    16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TPREL16,       16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TPREL16_LO,    16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TPREL16_HI,    16 },
+  { RELOC_POWERPC32::R_PPC_GOT_TPREL16_HA,    16 },
+  { RELOC_POWERPC32::R_PPC_GOT_DTPREL16,      16 },
+  { RELOC_POWERPC32::R_PPC_GOT_DTPREL16_LO,   16 },
+  { RELOC_POWERPC32::R_PPC_GOT_DTPREL16_HI,   16 },
+  { RELOC_POWERPC32::R_PPC_GOT_DTPREL16_HA,   16 },
+  { RELOC_POWERPC32::R_PPC_TLSGD,             32 },
+  { RELOC_POWERPC32::R_PPC_TLSLD,             32 },
+  { RELOC_POWERPC32::R_PPC_REL16,             16 },
+  { RELOC_POWERPC32::R_PPC_REL16_LO,          16 },
+  { RELOC_POWERPC32::R_PPC_REL16_HI,          16 },
+  { RELOC_POWERPC32::R_PPC_REL16_HA,          16 },
+};
+
+static const std::map<RELOC_POWERPC64, uint32_t> relocation_PPC64_sizes {
+  { RELOC_POWERPC64::R_PPC64_NONE,                 0 },
+  { RELOC_POWERPC64::R_PPC64_ADDR32,              32 },
+  { RELOC_POWERPC64::R_PPC64_ADDR24,              32 },
+  { RELOC_POWERPC64::R_PPC64_ADDR16,              16 },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_LO,           16 },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HI,           16 },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HA,           16 },
+  { RELOC_POWERPC64::R_PPC64_ADDR14,              32 },
+  { RELOC_POWERPC64::R_PPC64_ADDR14_BRTAKEN,      32 }, 
+  { RELOC_POWERPC64::R_PPC64_ADDR14_BRNTAKEN,     32 },
+  { RELOC_POWERPC64::R_PPC64_REL24,               32 },
+  { RELOC_POWERPC64::R_PPC64_REL14,               32 },
+  { RELOC_POWERPC64::R_PPC64_REL14_BRTAKEN,       32 },
+  { RELOC_POWERPC64::R_PPC64_REL14_BRNTAKEN,      32 },
+  { RELOC_POWERPC64::R_PPC64_GOT16,               16 },
+  { RELOC_POWERPC64::R_PPC64_GOT16_LO,            16 },
+  { RELOC_POWERPC64::R_PPC64_GOT16_HI,            16 },
+  { RELOC_POWERPC64::R_PPC64_GOT16_HA,            16 },
+  { RELOC_POWERPC64::R_PPC64_JMP_SLOT,            64 },
+  { RELOC_POWERPC64::R_PPC64_RELATIVE,            64 },
+  { RELOC_POWERPC64::R_PPC64_REL32,               32 },
+  { RELOC_POWERPC64::R_PPC64_ADDR64,              64 },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HIGHER,       16 },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HIGHERA,      16 },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HIGHEST,      16 },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HIGHESTA,     16 },
+  { RELOC_POWERPC64::R_PPC64_REL64,               64 },
+  { RELOC_POWERPC64::R_PPC64_TOC16,               16 },
+  { RELOC_POWERPC64::R_PPC64_TOC16_LO,            16 },
+  { RELOC_POWERPC64::R_PPC64_TOC16_HI,            16 },
+  { RELOC_POWERPC64::R_PPC64_TOC16_HA,            16 },
+  { RELOC_POWERPC64::R_PPC64_TOC,                 16 },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_DS,           16 },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_LO_DS,        16 },
+  { RELOC_POWERPC64::R_PPC64_GOT16_DS,            16 },
+  { RELOC_POWERPC64::R_PPC64_GOT16_LO_DS,         16 },
+  { RELOC_POWERPC64::R_PPC64_TOC16_DS,            16 },
+  { RELOC_POWERPC64::R_PPC64_TOC16_LO_DS,         16 },
+  { RELOC_POWERPC64::R_PPC64_TLS,                  0 },
+  { RELOC_POWERPC64::R_PPC64_DTPMOD64,            64 },
+  { RELOC_POWERPC64::R_PPC64_TPREL16,             16 },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_LO,          16 },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HI,          16 },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HA,          16 },
+  { RELOC_POWERPC64::R_PPC64_TPREL64,             64 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16,            16 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_LO,         16 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HI,         16 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HA,         16 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL64,            64 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSGD16,         16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSGD16_LO,      16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSGD16_HI,      16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSGD16_HA,      16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSLD16,         16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSLD16_LO,      16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSLD16_HI,      16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSLD16_HA,      16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TPREL16_DS,      16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TPREL16_LO_DS,   16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TPREL16_HI,      16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_TPREL16_HA,      16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_DTPREL16_DS,     16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_DTPREL16_LO_DS,  16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_DTPREL16_HI,     16 },
+  { RELOC_POWERPC64::R_PPC64_GOT_DTPREL16_HA,     16 },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_DS,          16 },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_LO_DS,       16 },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HIGHER,      16 },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HIGHERA,     16 },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HIGHEST,     16 },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HIGHESTA,    16 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_DS,         16 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_LO_DS,      16 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HIGHER,     16 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HIGHERA,    16 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HIGHEST,    16 },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HIGHESTA,   16 },
+  { RELOC_POWERPC64::R_PPC64_TLSGD,                0 },
+  { RELOC_POWERPC64::R_PPC64_TLSLD,                0 },
+  { RELOC_POWERPC64::R_PPC64_REL16,               16 },
+  { RELOC_POWERPC64::R_PPC64_REL16_LO,            16 },
+  { RELOC_POWERPC64::R_PPC64_REL16_HI,            16 },
+  { RELOC_POWERPC64::R_PPC64_REL16_HA,            16 },
+};
+
 }
 }
 
