@@ -26,14 +26,14 @@ namespace LIEF {
 namespace ELF {
 
 //! @brief Check if the given file is an ELF one.
-DLL_PUBLIC bool is_elf(const std::string& file);
+LIEF_API bool is_elf(const std::string& file);
 
 //! @brief check if the raw data is a ELF file
-DLL_PUBLIC bool is_elf(const std::vector<uint8_t>& raw);
+LIEF_API bool is_elf(const std::vector<uint8_t>& raw);
 
-DLL_PUBLIC unsigned long hash32(const char* name);
-DLL_PUBLIC unsigned long hash64(const char* name);
-DLL_PUBLIC uint32_t dl_new_hash(const char* name);
+LIEF_API unsigned long hash32(const char* name);
+LIEF_API unsigned long hash64(const char* name);
+LIEF_API uint32_t dl_new_hash(const char* name);
 
 
 }
