@@ -16,6 +16,9 @@
 #ifndef LIEF_PE_UNDEF_H_
 #define LIEF_PE_UNDEF_H_
 
+#ifdef IMAGE_FILE_MACHINE_UNKNOWN
+#pragma message("LIEF will disable #define enums. Please use LIEF_PE_XXXX or LIEF::PE::XXXX instead!")
+#endif
 
 #undef IMAGE_FILE_MACHINE_UNKNOWN
 #undef IMAGE_FILE_MACHINE_AM33
@@ -240,6 +243,11 @@
 #undef IMAGE_DEBUG_TYPE_OMAP_FROM_SRC
 #undef IMAGE_DEBUG_TYPE_BORLAND
 #undef IMAGE_DEBUG_TYPE_CLSID
+#undef IMAGE_DEBUG_TYPE_VC_FEATURE
+#undef IMAGE_DEBUG_TYPE_POGO
+#undef IMAGE_DEBUG_TYPE_ILTCG
+#undef IMAGE_DEBUG_TYPE_MPX
+#undef IMAGE_DEBUG_TYPE_REPRO
 
 #undef IMAGE_SCN_TYPE_NO_PAD
 #undef IMAGE_SCN_CNT_CODE
@@ -292,10 +300,12 @@
 #undef LANG_BASQUE
 #undef LANG_BELARUSIAN
 #undef LANG_BENGALI
+#undef LANG_BANGLA
 #undef LANG_BULGARIAN
 #undef LANG_CATALAN
 #undef LANG_CHINESE
 #undef LANG_CROATIAN
+#undef LANG_BOSNIAN
 #undef LANG_CZECH
 #undef LANG_DANISH
 #undef LANG_DIVEHI
@@ -362,7 +372,7 @@
 #undef LANG_MALTESE
 #undef LANG_MAORI
 #undef LANG_RHAETO_ROMANCE
-#undef LANG_SAAMI
+#undef LANG_SAMI
 #undef LANG_SORBIAN
 #undef LANG_SUTU
 #undef LANG_TSONGA
@@ -375,111 +385,247 @@
 #undef LANG_CORNISH
 #undef LANG_WELSH
 #undef LANG_BRETON
+#undef LANG_INUKTITUT
+#undef LANG_IRISH
+#undef LANG_LOWER_SORBIAN
+#undef LANG_PULAR
+#undef LANG_QUECHUA
+#undef LANG_TAMAZIGHT
+#undef LANG_TIGRINYA
+#undef LANG_VALENCIAN
 
 
-#undef SUBLANG_NEUTRAL
-#undef SUBLANG_DEFAULT
-#undef SUBLANG_SYS_DEFAULT
-#undef SUBLANG_ARABIC_SAUDI_ARABIA
-#undef SUBLANG_ARABIC_IRAQ
-#undef SUBLANG_ARABIC_EGYPT
-#undef SUBLANG_ARABIC_LIBYA
+
+#undef SUBLANG_AFRIKAANS_SOUTH_AFRICA
+#undef SUBLANG_ALBANIAN_ALBANIA
+#undef SUBLANG_ALSATIAN_FRANCE
+#undef SUBLANG_AMHARIC_ETHIOPIA
 #undef SUBLANG_ARABIC_ALGERIA
-#undef SUBLANG_ARABIC_MOROCCO
-#undef SUBLANG_ARABIC_TUNISIA
-#undef SUBLANG_ARABIC_OMAN
-#undef SUBLANG_ARABIC_YEMEN
-#undef SUBLANG_ARABIC_SYRIA
-#undef SUBLANG_ARABIC_JORDAN
-#undef SUBLANG_ARABIC_LEBANON
-#undef SUBLANG_ARABIC_KUWAIT
-#undef SUBLANG_ARABIC_UAE
 #undef SUBLANG_ARABIC_BAHRAIN
+#undef SUBLANG_ARABIC_EGYPT
+#undef SUBLANG_ARABIC_IRAQ
+#undef SUBLANG_ARABIC_JORDAN
+#undef SUBLANG_ARABIC_KUWAIT
+#undef SUBLANG_ARABIC_LEBANON
+#undef SUBLANG_ARABIC_LIBYA
+#undef SUBLANG_ARABIC_MOROCCO
+#undef SUBLANG_ARABIC_OMAN
 #undef SUBLANG_ARABIC_QATAR
-#undef SUBLANG_AZERI_LATIN
+#undef SUBLANG_ARABIC_SAUDI_ARABIA
+#undef SUBLANG_ARABIC_SYRIA
+#undef SUBLANG_ARABIC_TUNISIA
+#undef SUBLANG_ARABIC_UAE
+#undef SUBLANG_ARABIC_YEMEN
+#undef SUBLANG_ARMENIAN_ARMENIA
+#undef SUBLANG_ASSAMESE_INDIA
 #undef SUBLANG_AZERI_CYRILLIC
-#undef SUBLANG_CHINESE_TRADITIONAL
-#undef SUBLANG_CHINESE_SIMPLIFIED
+#undef SUBLANG_AZERI_LATIN
+#undef SUBLANG_BASHKIR_RUSSIA
+#undef SUBLANG_BASQUE_BASQUE
+#undef SUBLANG_BELARUSIAN_BELARUS
+#undef SUBLANG_BANGLA_BANGLADESH
+#undef SUBLANG_BANGLA_INDIA
+#undef SUBLANG_BOSNIAN_BOSNIA_HERZEGOVINA_CYRILLIC
+#undef SUBLANG_BOSNIAN_BOSNIA_HERZEGOVINA_LATIN
+#undef SUBLANG_BRETON_FRANCE
+#undef SUBLANG_BULGARIAN_BULGARIA
+#undef SUBLANG_CATALAN_CATALAN
 #undef SUBLANG_CHINESE_HONGKONG
-#undef SUBLANG_CHINESE_SINGAPORE
 #undef SUBLANG_CHINESE_MACAU
-#undef SUBLANG_DUTCH
+#undef SUBLANG_CHINESE_SIMPLIFIED
+#undef SUBLANG_CHINESE_SINGAPORE
+#undef SUBLANG_CHINESE_TRADITIONAL
+#undef SUBLANG_CORSICAN_FRANCE
+#undef SUBLANG_CROATIAN_BOSNIA_HERZEGOVINA_LATIN
+#undef SUBLANG_CROATIAN_CROATIA
+#undef SUBLANG_CUSTOM_DEFAULT
+#undef SUBLANG_CUSTOM_UNSPECIFIED
+#undef SUBLANG_CZECH_CZECH_REPUBLIC
+#undef SUBLANG_DANISH_DENMARK
+#undef SUBLANG_DARI_AFGHANISTAN
+#undef SUBLANG_DEFAULT
+#undef SUBLANG_DIVEHI_MALDIVES
 #undef SUBLANG_DUTCH_BELGIAN
-#undef SUBLANG_ENGLISH_US
-#undef SUBLANG_ENGLISH_UK
+#undef SUBLANG_DUTCH
 #undef SUBLANG_ENGLISH_AUS
-#undef SUBLANG_ENGLISH_CAN
-#undef SUBLANG_ENGLISH_NZ
-#undef SUBLANG_ENGLISH_EIRE
-#undef SUBLANG_ENGLISH_SOUTH_AFRICA
-#undef SUBLANG_ENGLISH_JAMAICA
-#undef SUBLANG_ENGLISH_CARIBBEAN
 #undef SUBLANG_ENGLISH_BELIZE
-#undef SUBLANG_ENGLISH_TRINIDAD
-#undef SUBLANG_ENGLISH_ZIMBABWE
+#undef SUBLANG_ENGLISH_CAN
+#undef SUBLANG_ENGLISH_CARIBBEAN
+#undef SUBLANG_ENGLISH_EIRE
+#undef SUBLANG_ENGLISH_INDIA
+#undef SUBLANG_ENGLISH_JAMAICA
+#undef SUBLANG_ENGLISH_MALAYSIA
+#undef SUBLANG_ENGLISH_NZ
 #undef SUBLANG_ENGLISH_PHILIPPINES
-#undef SUBLANG_FRENCH
+#undef SUBLANG_ENGLISH_SINGAPORE
+#undef SUBLANG_ENGLISH_SOUTH_AFRICA
+#undef SUBLANG_ENGLISH_TRINIDAD
+#undef SUBLANG_ENGLISH_UK
+#undef SUBLANG_ENGLISH_US
+#undef SUBLANG_ENGLISH_ZIMBABWE
+#undef SUBLANG_ENGLISH_IRELAND
+#undef SUBLANG_ESTONIAN_ESTONIA
+#undef SUBLANG_FAEROESE_FAROE_ISLANDS
+#undef SUBLANG_FILIPINO_PHILIPPINES
+#undef SUBLANG_FINNISH_FINLAND
 #undef SUBLANG_FRENCH_BELGIAN
 #undef SUBLANG_FRENCH_CANADIAN
-#undef SUBLANG_FRENCH_SWISS
 #undef SUBLANG_FRENCH_LUXEMBOURG
 #undef SUBLANG_FRENCH_MONACO
-#undef SUBLANG_GERMAN
-#undef SUBLANG_GERMAN_SWISS
+#undef SUBLANG_FRENCH_SWISS
+#undef SUBLANG_FRENCH
+#undef SUBLANG_FRISIAN_NETHERLANDS
+#undef SUBLANG_GALICIAN_GALICIAN
+#undef SUBLANG_GEORGIAN_GEORGIA
 #undef SUBLANG_GERMAN_AUSTRIAN
-#undef SUBLANG_GERMAN_LUXEMBOURG
 #undef SUBLANG_GERMAN_LIECHTENSTEIN
-#undef SUBLANG_ITALIAN
+#undef SUBLANG_GERMAN_LUXEMBOURG
+#undef SUBLANG_GERMAN_SWISS
+#undef SUBLANG_GERMAN
+#undef SUBLANG_GREEK_GREECE
+#undef SUBLANG_GREENLANDIC_GREENLAND
+#undef SUBLANG_GUJARATI_INDIA
+#undef SUBLANG_HAUSA_NIGERIA_LATIN
+#undef SUBLANG_HEBREW_ISRAEL
+#undef SUBLANG_HINDI_INDIA
+#undef SUBLANG_HUNGARIAN_HUNGARY
+#undef SUBLANG_ICELANDIC_ICELAND
+#undef SUBLANG_IGBO_NIGERIA
+#undef SUBLANG_INDONESIAN_INDONESIA
+#undef SUBLANG_INUKTITUT_CANADA_LATIN
+#undef SUBLANG_INUKTITUT_CANADA
+#undef SUBLANG_IRISH_IRELAND
 #undef SUBLANG_ITALIAN_SWISS
-#undef SUBLANG_KASHMIRI_SASIA
+#undef SUBLANG_ITALIAN
+#undef SUBLANG_JAPANESE_JAPAN
+#undef SUBLANG_KANNADA_INDIA
 #undef SUBLANG_KASHMIRI_INDIA
+#undef SUBLANG_KASHMIRI_SASIA
+#undef SUBLANG_KAZAK_KAZAKHSTAN
+#undef SUBLANG_KHMER_CAMBODIA
+#undef SUBLANG_KICHE_GUATEMALA
+#undef SUBLANG_KINYARWANDA_RWANDA
+#undef SUBLANG_KONKANI_INDIA
 #undef SUBLANG_KOREAN
+#undef SUBLANG_KYRGYZ_KYRGYZSTAN
+#undef SUBLANG_LAO_LAO
+#undef SUBLANG_LATVIAN_LATVIA
+#undef SUBLANG_LITHUANIAN_CLASSIC
 #undef SUBLANG_LITHUANIAN
-#undef SUBLANG_MALAY_MALAYSIA
+#undef SUBLANG_LOWER_SORBIAN_GERMANY
+#undef SUBLANG_LUXEMBOURGISH_LUXEMBOURG
+#undef SUBLANG_MACEDONIAN_MACEDONIA
 #undef SUBLANG_MALAY_BRUNEI_DARUSSALAM
+#undef SUBLANG_MALAY_MALAYSIA
+#undef SUBLANG_MALAYALAM_INDIA
+#undef SUBLANG_MALTESE_MALTA
+#undef SUBLANG_MAORI_NEW_ZEALAND
+#undef SUBLANG_MAPUDUNGUN_CHILE
+#undef SUBLANG_MARATHI_INDIA
+#undef SUBLANG_MOHAWK_MOHAWK
+#undef SUBLANG_MONGOLIAN_CYRILLIC_MONGOLIA
+#undef SUBLANG_MONGOLIAN_PRC
 #undef SUBLANG_NEPALI_INDIA
+#undef SUBLANG_NEPALI_NEPAL
+#undef SUBLANG_NEUTRAL
 #undef SUBLANG_NORWEGIAN_BOKMAL
 #undef SUBLANG_NORWEGIAN_NYNORSK
-#undef SUBLANG_PORTUGUESE
+#undef SUBLANG_OCCITAN_FRANCE
+#undef SUBLANG_ORIYA_INDIA
+#undef SUBLANG_PASHTO_AFGHANISTAN
+#undef SUBLANG_PERSIAN_IRAN
+#undef SUBLANG_POLISH_POLAND
 #undef SUBLANG_PORTUGUESE_BRAZILIAN
-#undef SUBLANG_SERBIAN_LATIN
+#undef SUBLANG_PORTUGUESE
+#undef SUBLANG_PUNJABI_INDIA
+#undef SUBLANG_QUECHUA_BOLIVIA
+#undef SUBLANG_QUECHUA_ECUADOR
+#undef SUBLANG_QUECHUA_PERU
+#undef SUBLANG_ROMANIAN_ROMANIA
+#undef SUBLANG_ROMANSH_SWITZERLAND
+#undef SUBLANG_RUSSIAN_RUSSIA
+#undef SUBLANG_SAMI_INARI_FINLAND
+#undef SUBLANG_SAMI_LULE_NORWAY
+#undef SUBLANG_SAMI_LULE_SWEDEN
+#undef SUBLANG_SAMI_NORTHERN_FINLAND
+#undef SUBLANG_SAMI_NORTHERN_NORWAY
+#undef SUBLANG_SAMI_NORTHERN_SWEDEN
+#undef SUBLANG_SAMI_SKOLT_FINLAND
+#undef SUBLANG_SAMI_SOUTHERN_NORWAY
+#undef SUBLANG_SAMI_SOUTHERN_SWEDEN
+#undef SUBLANG_SANSKRIT_INDIA
+#undef SUBLANG_SERBIAN_BOSNIA_HERZEGOVINA_CYRILLIC
+#undef SUBLANG_SERBIAN_BOSNIA_HERZEGOVINA_LATIN
+#undef SUBLANG_SERBIAN_CROATIA
 #undef SUBLANG_SERBIAN_CYRILLIC
-#undef SUBLANG_SPANISH
+#undef SUBLANG_SERBIAN_LATIN
+#undef SUBLANG_SINDHI_AFGHANISTAN
+#undef SUBLANG_SINDHI_INDIA
+#undef SUBLANG_SINDHI_PAKISTAN
+#undef SUBLANG_SINHALESE_SRI_LANKA
+#undef SUBLANG_SLOVAK_SLOVAKIA
+#undef SUBLANG_SLOVENIAN_SLOVENIA
+#undef SUBLANG_SOTHO_NORTHERN_SOUTH_AFRICA
+#undef SUBLANG_SPANISH_ARGENTINA
+#undef SUBLANG_SPANISH_BOLIVIA
+#undef SUBLANG_SPANISH_CHILE
+#undef SUBLANG_SPANISH_COLOMBIA
+#undef SUBLANG_SPANISH_COSTA_RICA
+#undef SUBLANG_SPANISH_DOMINICAN_REPUBLIC
+#undef SUBLANG_SPANISH_ECUADOR
+#undef SUBLANG_SPANISH_EL_SALVADOR
+#undef SUBLANG_SPANISH_GUATEMALA
+#undef SUBLANG_SPANISH_HONDURAS
 #undef SUBLANG_SPANISH_MEXICAN
 #undef SUBLANG_SPANISH_MODERN
-#undef SUBLANG_SPANISH_GUATEMALA
-#undef SUBLANG_SPANISH_COSTA_RICA
-#undef SUBLANG_SPANISH_PANAMA
-#undef SUBLANG_SPANISH_DOMINICAN_REPUBLIC
-#undef SUBLANG_SPANISH_VENEZUELA
-#undef SUBLANG_SPANISH_COLOMBIA
-#undef SUBLANG_SPANISH_PERU
-#undef SUBLANG_SPANISH_ARGENTINA
-#undef SUBLANG_SPANISH_ECUADOR
-#undef SUBLANG_SPANISH_CHILE
-#undef SUBLANG_SPANISH_URUGUAY
-#undef SUBLANG_SPANISH_PARAGUAY
-#undef SUBLANG_SPANISH_BOLIVIA
-#undef SUBLANG_SPANISH_EL_SALVADOR
-#undef SUBLANG_SPANISH_HONDURAS
 #undef SUBLANG_SPANISH_NICARAGUA
+#undef SUBLANG_SPANISH_PANAMA
+#undef SUBLANG_SPANISH_PARAGUAY
+#undef SUBLANG_SPANISH_PERU
 #undef SUBLANG_SPANISH_PUERTO_RICO
-#undef SUBLANG_SWEDISH
+#undef SUBLANG_SPANISH_URUGUAY
+#undef SUBLANG_SPANISH_US
+#undef SUBLANG_SPANISH_VENEZUELA
+#undef SUBLANG_SPANISH
+#undef SUBLANG_SWAHILI_KENYA
 #undef SUBLANG_SWEDISH_FINLAND
-#undef SUBLANG_URDU_PAKISTAN
+#undef SUBLANG_SWEDISH
+#undef SUBLANG_SYRIAC_SYRIA
+#undef SUBLANG_SYS_DEFAULT
+#undef SUBLANG_TAJIK_TAJIKISTAN
+#undef SUBLANG_TAMAZIGHT_ALGERIA_LATIN
+#undef SUBLANG_TAMIL_INDIA
+#undef SUBLANG_TATAR_RUSSIA
+#undef SUBLANG_TELUGU_INDIA
+#undef SUBLANG_THAI_THAILAND
+#undef SUBLANG_TIBETAN_PRC
+#undef SUBLANG_TIGRIGNA_ERITREA
+#undef SUBLANG_TSWANA_SOUTH_AFRICA
+#undef SUBLANG_TURKISH_TURKEY
+#undef SUBLANG_TURKMEN_TURKMENISTAN
+#undef SUBLANG_UI_CUSTOM_DEFAULT
+#undef SUBLANG_UIGHUR_PRC
+#undef SUBLANG_UKRAINIAN_UKRAINE
+#undef SUBLANG_UPPER_SORBIAN_GERMANY
 #undef SUBLANG_URDU_INDIA
-#undef SUBLANG_UZBEK_LATIN
+#undef SUBLANG_URDU_PAKISTAN
 #undef SUBLANG_UZBEK_CYRILLIC
-#undef SUBLANG_DUTCH_SURINAM
-#undef SUBLANG_ROMANIAN
-#undef SUBLANG_ROMANIAN_MOLDAVIA
-#undef SUBLANG_RUSSIAN
-#undef SUBLANG_RUSSIAN_MOLDAVIA
-#undef SUBLANG_CROATIAN
-#undef SUBLANG_LITHUANIAN_CLASSIC
-#undef SUBLANG_GAELIC
-#undef SUBLANG_GAELIC_SCOTTISH
-#undef SUBLANG_GAELIC_MANX
+#undef SUBLANG_UZBEK_LATIN
+#undef SUBLANG_VIETNAMESE_VIETNAM
+#undef SUBLANG_WELSH_UNITED_KINGDOM
+#undef SUBLANG_WOLOF_SENEGAL
+#undef SUBLANG_XHOSA_SOUTH_AFRICA
+#undef SUBLANG_YAKUT_RUSSIA
+#undef SUBLANG_YI_PRC
+#undef SUBLANG_YORUBA_NIGERIA
+#undef SUBLANG_ZULU_SOUTH_AFRICA
+#undef SUBLANG_PULAR_SENEGAL
+#undef SUBLANG_PUNJABI_PAKISTAN
+#undef SUBLANG_TSWANA_BOTSWANA
+#undef SUBLANG_TAMIL_SRI_LANKA
+#undef SUBLANG_TIGRINYA_ETHIOPIA
+#undef SUBLANG_TIGRINYA_ERITREA
+#undef SUBLANG_VALENCIAN_VALENCIA
 
 #undef WS_EX_DLGMODALFRAME
 #undef WS_EX_NOPARENTNOTIFY
@@ -586,4 +732,5 @@
 #undef VFT2_UNKNOWN
 
 
+
 #endif
