@@ -16,6 +16,7 @@
 #include <algorithm>
 #include <iterator>
 #include <numeric>
+#include <sstream>
 
 #if defined(__unix__) || (defined(__APPLE__) && defined(__MACH__))
 #include <unistd.h>
@@ -25,7 +26,7 @@
 
 #include <stdexcept>
 
-#include "easylogging++.h"
+#include "LIEF/logging++.hpp"
 
 #include "LIEF/exception.hpp"
 #include "LIEF/utils.hpp"
@@ -34,7 +35,10 @@
 #include "LIEF/ELF/Binary.hpp"
 #include "LIEF/ELF/Builder.hpp"
 
+#include "LIEF/ELF/hash.hpp"
+
 #include "Binary.tcc"
+#include "Object.tcc"
 
 namespace LIEF {
 namespace ELF {
@@ -64,7 +68,7 @@ ELF_CLASS Binary::type(void) const {
 }
 
 size_t Binary::hash(const std::string& name) {
-  if (this->type_ == ELFCLASS32) {
+  if (this->type_ == ELF_CLASS::ELFCLASS32) {
     return hash32(name.c_str());
   } else {
     return hash32(name.c_str());
@@ -213,8 +217,8 @@ DynamicEntry& Binary::add(const DynamicEntry& entry) {
 
 
 Note& Binary::add(const Note& note) {
-  this->notes_.emplace_back(note);
-  return this->notes_.back();
+  this->notes_.emplace_back(new Note{note});
+  return *this->notes_.back();
 }
 
 
@@ -276,7 +280,7 @@ void Binary::remove(const Section& section, bool clear) {
   }
 
   if (clear) {
-    s->content(std::vector<uint8_t>(s->size(), 0));
+    s->clear(0);
   }
 
   this->datahandler_->remove(s->file_offset(), s->size(), DataHandler::Node::SECTION);
@@ -297,22 +301,24 @@ void Binary::remove(const Note& note) {
   auto&& it_note = std::find_if(
       std::begin(this->notes_),
       std::end(this->notes_),
-      [&note] (const Note& n)
+      [&note] (const Note* n)
       {
-        return note == n;
+        return note == *n;
       });
 
   if (it_note == std::end(this->notes_)) {
     throw not_found(std::string("Can't find note '") + to_string(static_cast<NOTE_TYPES>(note.type())) +  "'!");
   }
-
+  delete *it_note;
   this->notes_.erase(it_note);
 }
 
 void Binary::remove(NOTE_TYPES type) {
   for (auto&& it = std::begin(this->notes_);
               it != std::end(this->notes_);) {
-    if (it->type() == type) {
+    Note* n = *it;
+    if (static_cast<NOTE_TYPES>(n->type()) == type) {
+      delete n;
       it = this->notes_.erase(it);
     } else {
       ++it;
@@ -349,6 +355,183 @@ it_const_symbols Binary::dynamic_symbols(void) const {
 
 
 it_symbols Binary::symbols(void) {
+  return this->static_dyn_symbols();
+}
+
+it_const_symbols Binary::symbols(void) const {
+  return this->static_dyn_symbols();
+}
+
+
+Symbol& Binary::export_symbol(const Symbol& symbol) {
+
+  // Chck if the symbol is in the dynamic symbol table
+  auto&& it_symbol = std::find_if(
+      std::begin(this->dynamic_symbols_),
+      std::end(this->dynamic_symbols_),
+      [&symbol] (const Symbol* s) {
+        return *s == symbol;
+      });
+
+  if (it_symbol == std::end(this->dynamic_symbols_)) {
+    // Create a new one
+    Symbol& new_sym = this->add_dynamic_symbol(symbol, SymbolVersion::local());
+    return this->export_symbol(new_sym);
+  }
+
+  auto&& it_text = std::find_if(
+      std::begin(this->sections_),
+      std::end(this->sections_),
+      [] (const Section* s) {
+        return s->name() == ".text";
+      });
+  size_t text_idx = std::distance(std::begin(this->sections_), it_text);
+
+  Symbol& s = **it_symbol;
+  if (s.binding() != SYMBOL_BINDINGS::STB_WEAK or s.binding() != SYMBOL_BINDINGS::STB_GLOBAL) {
+    s.binding(SYMBOL_BINDINGS::STB_GLOBAL);
+  }
+
+  if (s.type() == ELF_SYMBOL_TYPES::STT_NOTYPE) {
+    s.type(ELF_SYMBOL_TYPES::STT_COMMON);
+  }
+
+  if (s.shndx() == 0) {
+    s.shndx(text_idx);
+  }
+
+  s.visibility(ELF_SYMBOL_VISIBILITY::STV_DEFAULT);
+  return s;
+}
+
+Symbol& Binary::export_symbol(const std::string& symbol_name, uint64_t value) {
+  if (this->has_dynamic_symbol(symbol_name)) {
+    Symbol& s = this->get_dynamic_symbol(symbol_name);
+    if (value > 0) {
+      s.value(value);
+    }
+    return this->export_symbol(s);
+  }
+
+  if (this->has_static_symbol(symbol_name)) {
+    Symbol& s = this->get_static_symbol(symbol_name);
+    if (value > 0) {
+      s.value(value);
+    }
+    return this->export_symbol(s);
+  }
+
+  // Create a new one
+  Symbol newsym;
+  newsym.name(symbol_name);
+  newsym.type(ELF_SYMBOL_TYPES::STT_COMMON);
+  newsym.binding(SYMBOL_BINDINGS::STB_GLOBAL);
+  newsym.visibility(ELF_SYMBOL_VISIBILITY::STV_DEFAULT);
+  newsym.value(value);
+  return this->export_symbol(newsym);
+}
+
+
+Symbol& Binary::add_exported_function(uint64_t address, const std::string& name) {
+  std::string funcname = name;
+  if (funcname.size() == 0) {
+    std::stringstream ss;
+    ss << "func_" << std::hex << address;
+    funcname = ss.str();
+  }
+
+  // First: Check if a symbol with the given 'name' exists in the **dynamic** table
+  if (this->has_dynamic_symbol(funcname)) {
+    Symbol& s = this->get_dynamic_symbol(funcname);
+    s.type(ELF_SYMBOL_TYPES::STT_FUNC);
+    s.binding(SYMBOL_BINDINGS::STB_GLOBAL);
+    s.visibility(ELF_SYMBOL_VISIBILITY::STV_DEFAULT);
+    s.value(address);
+    return this->export_symbol(s);
+  }
+
+  // Second: Check if a symbol with the given 'name' exists in the **static**
+  if (this->has_static_symbol(funcname)) {
+    Symbol& s = this->get_static_symbol(funcname);
+    s.type(ELF_SYMBOL_TYPES::STT_FUNC);
+    s.binding(SYMBOL_BINDINGS::STB_GLOBAL);
+    s.visibility(ELF_SYMBOL_VISIBILITY::STV_DEFAULT);
+    s.value(address);
+    return this->export_symbol(s);
+  }
+
+  // Create a new Symbol
+  Symbol funcsym;
+  funcsym.name(funcname);
+  funcsym.type(ELF_SYMBOL_TYPES::STT_FUNC);
+  funcsym.binding(SYMBOL_BINDINGS::STB_GLOBAL);
+  funcsym.visibility(ELF_SYMBOL_VISIBILITY::STV_DEFAULT);
+  funcsym.value(address);
+
+  return this->export_symbol(funcsym);
+
+}
+
+
+bool Binary::has_dynamic_symbol(const std::string& name) const {
+  auto&& it_symbol = std::find_if(
+      std::begin(this->dynamic_symbols_),
+      std::end(this->dynamic_symbols_),
+      [&name] (const Symbol* s) {
+        return s->name() == name;
+      });
+  return it_symbol != std::end(this->dynamic_symbols_);
+}
+
+const Symbol& Binary::get_dynamic_symbol(const std::string& name) const {
+  if (not this->has_dynamic_symbol(name)) {
+    throw not_found("Symbol '" + name + "' not found!");
+  }
+
+  auto&& it_symbol = std::find_if(
+      std::begin(this->dynamic_symbols_),
+      std::end(this->dynamic_symbols_),
+      [&name] (const Symbol* s) {
+        return s->name() == name;
+      });
+  return **it_symbol;
+}
+
+Symbol& Binary::get_dynamic_symbol(const std::string& name) {
+  return const_cast<Symbol&>(static_cast<const Binary*>(this)->get_dynamic_symbol(name));
+}
+
+bool Binary::has_static_symbol(const std::string& name) const {
+  auto&& it_symbol = std::find_if(
+      std::begin(this->static_symbols_),
+      std::end(this->static_symbols_),
+      [&name] (const Symbol* s) {
+        return s->name() == name;
+      });
+  return it_symbol != std::end(this->static_symbols_);
+}
+
+const Symbol& Binary::get_static_symbol(const std::string& name) const {
+  if (not this->has_static_symbol(name)) {
+    throw not_found("Symbol '" + name + "' not found!");
+  }
+
+  auto&& it_symbol = std::find_if(
+      std::begin(this->static_symbols_),
+      std::end(this->static_symbols_),
+      [&name] (const Symbol* s) {
+        return s->name() == name;
+      });
+  return **it_symbol;
+
+}
+
+Symbol& Binary::get_static_symbol(const std::string& name) {
+  return const_cast<Symbol&>(static_cast<const Binary*>(this)->get_static_symbol(name));
+}
+
+
+symbols_t Binary::static_dyn_symbols(void) const {
   symbols_t symbols;
   symbols.reserve(this->dynamic_symbols().size() + this->static_symbols().size());
   for (Symbol& s : this->dynamic_symbols()) {
@@ -358,37 +541,21 @@ it_symbols Binary::symbols(void) {
   for (Symbol& s : this->static_symbols()) {
     symbols.push_back(&s);
   }
-
-  return it_symbols{symbols};
-}
-
-it_const_symbols Binary::symbols(void) const {
-  symbols_t symbols;
-
-  symbols.reserve(this->dynamic_symbols().size() + this->static_symbols().size());
-
-  for (const Symbol& s : this->dynamic_symbols()) {
-    symbols.push_back(const_cast<Symbol*>(&s));
-  }
-
-  for (const Symbol& s : this->static_symbols()) {
-    symbols.push_back(const_cast<Symbol*>(&s));
-  }
-
-  return it_const_symbols{symbols};
+  return symbols;
 }
 
 // Exported
 // --------
 
 it_exported_symbols Binary::exported_symbols(void) {
-  return {this->dynamic_symbols_,
+
+  return {this->static_dyn_symbols(),
     [] (const Symbol* symbol) { return symbol->is_exported(); }
   };
 }
 
 it_const_exported_symbols Binary::exported_symbols(void) const {
-  return {this->dynamic_symbols_,
+  return {this->static_dyn_symbols(),
     [] (const Symbol* symbol) { return symbol->is_exported(); }
   };
 }
@@ -399,13 +566,13 @@ it_const_exported_symbols Binary::exported_symbols(void) const {
 // --------
 
 it_imported_symbols Binary::imported_symbols(void) {
-  return filter_iterator<symbols_t>{std::ref(this->dynamic_symbols_),
+  return {this->static_dyn_symbols(),
     [] (const Symbol* symbol) { return symbol->is_imported(); }
   };
 }
 
 it_const_imported_symbols Binary::imported_symbols(void) const {
-  return const_filter_iterator<symbols_t>{std::cref(this->dynamic_symbols_),
+  return {this->static_dyn_symbols(),
     [] (const Symbol* symbol) { return symbol->is_imported(); }
   };
 }
@@ -596,6 +763,18 @@ Relocation& Binary::add_dynamic_relocation(const Relocation& relocation) {
   relocation_ptr->purpose(RELOCATION_PURPOSES::RELOC_PURPOSE_DYNAMIC);
   relocation_ptr->architecture_ = this->header().machine_type();
   this->relocations_.push_back(relocation_ptr);
+
+  // Update the Dynamic Section (Thanks to @yd0b0N)
+  bool is_rela = relocation.is_rela();
+  DYNAMIC_TAGS tag_sz  = is_rela ? DYNAMIC_TAGS::DT_RELASZ  : DYNAMIC_TAGS::DT_RELSZ;
+  DYNAMIC_TAGS tag_ent = is_rela ? DYNAMIC_TAGS::DT_RELAENT : DYNAMIC_TAGS::DT_RELENT;
+
+  if (this->has(tag_sz) and this->has(tag_ent)) {
+   DynamicEntry &dt_sz  = this->get(tag_sz);
+   DynamicEntry &dt_ent = this->get(tag_ent);
+   dt_sz.value(dt_sz.value() + dt_ent.value());
+  }
+
   return *relocation_ptr;
 }
 
@@ -992,7 +1171,7 @@ Segment& Binary::extend(const Segment& segment, uint64_t size) {
     case SEGMENT_TYPES::PT_PHDR:
     case SEGMENT_TYPES::PT_LOAD:
       {
-        return this->extend_segment<PT_LOAD>(segment, size);
+        return this->extend_segment<SEGMENT_TYPES::PT_LOAD>(segment, size);
         break;
       }
 
@@ -1065,7 +1244,7 @@ Section& Binary::extend(const Section& section, uint64_t size) {
 // Patch
 // =====
 
-void Binary::patch_address(uint64_t address, const std::vector<uint8_t>& patch_value) {
+void Binary::patch_address(uint64_t address, const std::vector<uint8_t>& patch_value, LIEF::Binary::VA_TYPES) {
   // Find the segment associated with the virtual address
   Segment& segment_topatch = this->segment_from_virtual_address(address);
   const uint64_t offset = address - segment_topatch.virtual_address();
@@ -1081,7 +1260,7 @@ void Binary::patch_address(uint64_t address, const std::vector<uint8_t>& patch_v
 }
 
 
-void Binary::patch_address(uint64_t address, uint64_t patch_value, size_t size) {
+void Binary::patch_address(uint64_t address, uint64_t patch_value, size_t size, LIEF::Binary::VA_TYPES) {
   if (size > sizeof(patch_value)) {
     throw std::runtime_error("Invalid size (" + std::to_string(size) + ")");
   }
@@ -1138,7 +1317,7 @@ const Segment& Binary::segment_from_virtual_address(uint64_t address) const {
           return false;
         }
         return ((segment->virtual_address() <= address) and
-            (segment->virtual_address() + segment->virtual_size()) >= address);
+            (segment->virtual_address() + segment->virtual_size()) > address);
       });
 
   if (it_segment == this->segments_.cend()) {
@@ -1195,6 +1374,17 @@ bool Binary::has_section(const std::string& name) const {
 
 void Binary::strip(void) {
   this->static_symbols_ = {};
+
+  //for (Section* sec : this->sections_) {
+  //  if (sec->segments().size() == 0 and sec->name() != ".shstrtab" and sec->type() != ELF_SECTION_TYPES::SHT_NULL) {
+  //    this->remove(*sec, /* clear */ true);
+  //    return strip();
+  //  }
+  //}
+  if (this->has(ELF_SECTION_TYPES::SHT_SYMTAB)) {
+    Section& symtab = this->get(ELF_SECTION_TYPES::SHT_SYMTAB);
+    this->remove(symtab, /* clear */ true);
+  }
 }
 
 
@@ -1204,8 +1394,13 @@ Symbol& Binary::add_static_symbol(const Symbol& symbol) {
 }
 
 
-Symbol& Binary::add_dynamic_symbol(const Symbol& symbol) {
-  this->dynamic_symbols_.push_back(new Symbol{symbol});
+Symbol& Binary::add_dynamic_symbol(const Symbol& symbol, const SymbolVersion& version) {
+  Symbol* sym = new Symbol{symbol};
+  SymbolVersion* symver = new SymbolVersion{version};
+  sym->symbol_version_ = symver;
+
+  this->dynamic_symbols_.push_back(sym);
+  this->symbol_version_table_.push_back(symver);
   return *(this->dynamic_symbols_.back());
 }
 
@@ -1321,8 +1516,9 @@ Section& Binary::section_from_virtual_address(uint64_t address) {
   return const_cast<Section&>(static_cast<const Binary*>(this)->section_from_virtual_address(address));
 }
 
-std::vector<uint8_t> Binary::get_content_from_virtual_address(uint64_t virtual_address, uint64_t size) const {
+std::vector<uint8_t> Binary::get_content_from_virtual_address(uint64_t virtual_address, uint64_t size, LIEF::Binary::VA_TYPES) const {
   const Segment& segment = this->segment_from_virtual_address(virtual_address);
+
   const std::vector<uint8_t>& content = segment.content();
   const uint64_t offset = virtual_address - segment.virtual_address();
   uint64_t checked_size = size;
@@ -1399,12 +1595,12 @@ const Note& Binary::get(NOTE_TYPES type) const {
   auto&& it_note = std::find_if(
       std::begin(this->notes_),
       std::end(this->notes_),
-      [type] (const Note& note)
+      [type] (const Note* note)
       {
-        return note.type() == type;
+        return static_cast<NOTE_TYPES>(note->type()) == type;
       });
 
-  return *it_note;
+  return **it_note;
 }
 
 
@@ -1412,6 +1608,29 @@ Note& Binary::get(NOTE_TYPES type) {
   return const_cast<Note&>(static_cast<const Binary*>(this)->get(type));
 }
 
+const Section& Binary::get(ELF_SECTION_TYPES type) const {
+
+  if (not this->has(type)) {
+    throw not_found("Unable to find a section of type '" + std::string(to_string(type)) + "'.");
+  }
+
+  auto&& it_section = std::find_if(
+      std::begin(this->sections_),
+      std::end(this->sections_),
+      [type] (const Section* section)
+      {
+        return section->type() == type;
+      });
+
+  return **it_section;
+}
+
+
+Section& Binary::get(ELF_SECTION_TYPES type) {
+  return const_cast<Section&>(static_cast<const Binary*>(this)->get(type));
+}
+
+
 
 bool Binary::has(SEGMENT_TYPES type) const {
   auto&& it_segment = std::find_if(
@@ -1430,14 +1649,27 @@ bool Binary::has(NOTE_TYPES type) const {
   auto&& it_note = std::find_if(
       std::begin(this->notes_),
       std::end(this->notes_),
-      [type] (const Note& note)
+      [type] (const Note* note)
       {
-        return note.type() == type;
+        return static_cast<NOTE_TYPES>(note->type()) == type;
       });
 
   return it_note != std::end(this->notes_);
 }
 
+bool Binary::has(ELF_SECTION_TYPES type) const {
+  auto&& it_section = std::find_if(
+      std::begin(this->sections_),
+      std::end(this->sections_),
+      [type] (const Section* section)
+      {
+        return section->type() == type;
+      });
+
+  return it_section != std::end(this->sections_);
+}
+
+
 
 void Binary::permute_dynamic_symbols(const std::vector<size_t>& permutation) {
   std::set<size_t> done;
@@ -1493,15 +1725,16 @@ bool Binary::has_notes(void) const {
 }
 
 it_const_notes Binary::notes(void) const {
-  return {this->notes_};
+  return this->notes_;
 }
 
 it_notes Binary::notes(void) {
-  return {this->notes_};
+  return this->notes_;
 }
 
 
-void Binary::accept(LIEF::Visitor&) const {
+void Binary::accept(LIEF::Visitor& visitor) const {
+  visitor.visit(*this);
 }
 
 bool Binary::use_gnu_hash(void) const {
@@ -1610,7 +1843,7 @@ void Binary::shift_dynamic_entries(uint64_t from, uint64_t shift) {
       case DYNAMIC_TAGS::DT_FINI_ARRAY:
       case DYNAMIC_TAGS::DT_PREINIT_ARRAY:
         {
-          std::vector<uint64_t>& array = entry->array();
+          DynamicEntryArray::array_t& array = entry->as<DynamicEntryArray>()->array();
           for (uint64_t& address : array) {
             if (address >= from) {
               if (
@@ -1832,7 +2065,6 @@ const DynamicEntry& Binary::operator[](DYNAMIC_TAGS tag) const {
   return this->get(tag);
 }
 
-
 Note& Binary::operator[](NOTE_TYPES type) {
   return this->get(type);
 }
@@ -1841,6 +2073,24 @@ const Note& Binary::operator[](NOTE_TYPES type) const {
   return this->get(type);
 }
 
+Section& Binary::operator[](ELF_SECTION_TYPES type) {
+  return this->get(type);
+}
+
+const Section& Binary::operator[](ELF_SECTION_TYPES type) const {
+  return this->get(type);
+}
+
+bool Binary::operator==(const Binary& rhs) const {
+  size_t hash_lhs = Hash::hash(*this);
+  size_t hash_rhs = Hash::hash(rhs);
+  return hash_lhs == hash_rhs;
+}
+
+bool Binary::operator!=(const Binary& rhs) const {
+  return not (*this == rhs);
+}
+
 std::ostream& Binary::print(std::ostream& os) const {
 
   os << "Header" << std::endl;
@@ -1988,7 +2238,6 @@ std::ostream& Binary::print(std::ostream& os) const {
 
 
 Binary::~Binary(void) {
-
   for (Relocation* relocation : this->relocations_) {
     delete relocation;
   }
@@ -2025,6 +2274,10 @@ Binary::~Binary(void) {
     delete svr;
   }
 
+  for (Note* note : this->notes_) {
+    delete note;
+  }
+
   delete datahandler_;
 }
 
