@@ -15,13 +15,14 @@
  */
 #include "LIEF/ELF/Structures.hpp"
 #include "LIEF/ELF/EnumToString.hpp"
+#include "frozen.hpp"
 #include <map>
 
 namespace LIEF {
 namespace ELF {
 
 const char* to_string(SYMBOL_BINDINGS e) {
-  const std::map<SYMBOL_BINDINGS, const char*> enumStrings {
+  CONST_MAP(SYMBOL_BINDINGS, const char*, 8) enumStrings {
     { SYMBOL_BINDINGS::STB_LOCAL,      "LOCAL" },
     { SYMBOL_BINDINGS::STB_GLOBAL,     "GLOBAL" },
     { SYMBOL_BINDINGS::STB_WEAK,       "WEAK" },
@@ -37,7 +38,7 @@ const char* to_string(SYMBOL_BINDINGS e) {
 
 
 const char* to_string(E_TYPE e) {
-  const std::map<E_TYPE, const char*> enumStrings {
+  CONST_MAP(E_TYPE, const char*, 7) enumStrings {
     { E_TYPE::ET_NONE,   "NONE" },
     { E_TYPE::ET_REL,    "RELOCATABLE" },
     { E_TYPE::ET_EXEC,   "EXECUTABLE" },
@@ -52,7 +53,7 @@ const char* to_string(E_TYPE e) {
 
 
 const char* to_string(VERSION e) {
-  const std::map<VERSION, const char*> enumStrings {
+  CONST_MAP(VERSION, const char*, 2) enumStrings {
     { VERSION::EV_NONE,    "NONE" },
     { VERSION::EV_CURRENT, "CURRENT" }
   };
@@ -62,7 +63,7 @@ const char* to_string(VERSION e) {
 
 
 const char* to_string(ARCH e) {
-  const std::map<ARCH, const char*> enumStrings {
+  CONST_MAP(ARCH, const char*, 175) enumStrings {
     { ARCH::EM_NONE,          "None" },
     { ARCH::EM_M32,           "M32"},
     { ARCH::EM_SPARC,         "SPARC"},
@@ -244,7 +245,7 @@ const char* to_string(ARCH e) {
 }
 
 const char* to_string(SEGMENT_TYPES e) {
-  const std::map<SEGMENT_TYPES, const char*> enumStrings {
+  CONST_MAP(SEGMENT_TYPES, const char*, 24) enumStrings {
     { SEGMENT_TYPES::PT_NULL,          "NULL" },
     { SEGMENT_TYPES::PT_LOAD,          "LOAD" },
     { SEGMENT_TYPES::PT_DYNAMIC,       "DYNAMIC" },
@@ -275,7 +276,7 @@ const char* to_string(SEGMENT_TYPES e) {
 }
 
 const char* to_string(DYNAMIC_TAGS e) {
-  const std::map<DYNAMIC_TAGS, const char*> enumStrings {
+  CONST_MAP(DYNAMIC_TAGS, const char*, 91) enumStrings {
     { DYNAMIC_TAGS::DT_NULL,                       "NULL"},
     { DYNAMIC_TAGS::DT_NEEDED,                     "NEEDED"},
     { DYNAMIC_TAGS::DT_PLTRELSZ,                   "PLTRELSZ"},
@@ -375,7 +376,7 @@ const char* to_string(DYNAMIC_TAGS e) {
 
 
 const char* to_string(ELF_SECTION_TYPES e) {
-  const std::map<ELF_SECTION_TYPES, const char*> enumStrings {
+  CONST_MAP(ELF_SECTION_TYPES, const char*, 38) enumStrings {
     { ELF_SECTION_TYPES::SHT_NULL,               "NULL"},
     { ELF_SECTION_TYPES::SHT_PROGBITS,           "PROGBITS"},
     { ELF_SECTION_TYPES::SHT_SYMTAB,             "SYMTAB"},
@@ -420,7 +421,7 @@ const char* to_string(ELF_SECTION_TYPES e) {
 }
 
 const char* to_string(ELF_SECTION_FLAGS e) {
-  const std::map<ELF_SECTION_FLAGS, const char*> enumStrings {
+  CONST_MAP(ELF_SECTION_FLAGS, const char*, 25) enumStrings {
     { ELF_SECTION_FLAGS::SHF_NONE,             "NONE"},
     { ELF_SECTION_FLAGS::SHF_WRITE,            "WRITE"},
     { ELF_SECTION_FLAGS::SHF_ALLOC,            "ALLOC"},
@@ -453,7 +454,7 @@ const char* to_string(ELF_SECTION_FLAGS e) {
 
 
 const char* to_string(ELF_SYMBOL_TYPES e) {
-  const std::map<ELF_SYMBOL_TYPES, const char*> enumStrings {
+  CONST_MAP(ELF_SYMBOL_TYPES, const char*, 12) enumStrings {
     { ELF_SYMBOL_TYPES::STT_NOTYPE,    "NOTYPE"},
     { ELF_SYMBOL_TYPES::STT_OBJECT,    "OBJECT"},
     { ELF_SYMBOL_TYPES::STT_FUNC,      "FUNC"},
@@ -473,7 +474,7 @@ const char* to_string(ELF_SYMBOL_TYPES e) {
 
 
 const char* to_string(RELOC_x86_64 e) {
-  const std::map<RELOC_x86_64, const char*> enumStrings {
+  CONST_MAP(RELOC_x86_64, const char*, 43) enumStrings {
     { RELOC_x86_64::R_X86_64_NONE,            "NONE"},
     { RELOC_x86_64::R_X86_64_64,              "R64"},
     { RELOC_x86_64::R_X86_64_PC32,            "PC32"},
@@ -511,14 +512,19 @@ const char* to_string(RELOC_x86_64 e) {
     { RELOC_x86_64::R_X86_64_GOTPC32_TLSDESC, "GOTPC32_TLSDESC"},
     { RELOC_x86_64::R_X86_64_TLSDESC_CALL,    "TLSDESC_CALL"},
     { RELOC_x86_64::R_X86_64_TLSDESC,         "TLSDESC"},
-    { RELOC_x86_64::R_X86_64_IRELATIVE,       "IRELATIVE"}
+    { RELOC_x86_64::R_X86_64_IRELATIVE,       "IRELATIVE"},
+    { RELOC_x86_64::R_X86_64_RELATIVE64,      "RELATIVE64"},
+    { RELOC_x86_64::R_X86_64_PC32_BND,        "PC32_BND"},
+    { RELOC_x86_64::R_X86_64_PLT32_BND,       "PLT32_BND"},
+    { RELOC_x86_64::R_X86_64_GOTPCRELX,       "GOTPCRELX"},
+    { RELOC_x86_64::R_X86_64_REX_GOTPCRELX,   "REX_GOTPCRELX"},
   };
   auto   it  = enumStrings.find(e);
   return it == enumStrings.end() ? "UNDEFINED" : it->second;
 }
 
 const char* to_string(RELOC_ARM e) {
-  const std::map<RELOC_ARM, const char*> enumStrings {
+  CONST_MAP(RELOC_ARM, const char*, 138) enumStrings {
     { RELOC_ARM::R_ARM_NONE,               "NONE"},
     { RELOC_ARM::R_ARM_PC24,               "PC24"},
     { RELOC_ARM::R_ARM_ABS32,              "ABS32"},
@@ -663,7 +669,7 @@ const char* to_string(RELOC_ARM e) {
 }
 
 const char* to_string(RELOC_AARCH64 e) {
-  const std::map<RELOC_AARCH64, const char*> enumStrings {
+  CONST_MAP(RELOC_AARCH64, const char*, 123) enumStrings {
     { RELOC_AARCH64::R_AARCH64_NONE,                         "NONE"},
     { RELOC_AARCH64::R_AARCH64_ABS64,                        "ABS64"},
     { RELOC_AARCH64::R_AARCH64_ABS32,                        "ABS32"},
@@ -795,7 +801,7 @@ const char* to_string(RELOC_AARCH64 e) {
 
 
 const char* to_string(RELOC_i386 e) {
-  const std::map<RELOC_i386, const char*> enumStrings {
+  CONST_MAP(RELOC_i386, const char*, 41) enumStrings {
     { RELOC_i386::R_386_NONE,          "NONE"},
     { RELOC_i386::R_386_32,            "R32"},
     { RELOC_i386::R_386_PC32,          "PC32"},
@@ -842,8 +848,163 @@ const char* to_string(RELOC_i386 e) {
   return it == enumStrings.end() ? "UNDEFINED" : it->second;
 }
 
+const char* to_string(RELOC_POWERPC32 e) {
+  CONST_MAP(RELOC_POWERPC32, const char*, 57) enumStrings {
+  { RELOC_POWERPC32::R_PPC_NONE,              "NONE" },
+  { RELOC_POWERPC32::R_PPC_ADDR32,            "ADDR32" },
+  { RELOC_POWERPC32::R_PPC_ADDR24,            "ADDR24" },
+  { RELOC_POWERPC32::R_PPC_ADDR16,            "ADDR16" },
+  { RELOC_POWERPC32::R_PPC_ADDR16_LO,         "ADDR16" },
+  { RELOC_POWERPC32::R_PPC_ADDR16_HI,         "ADDR16_HI" },
+  { RELOC_POWERPC32::R_PPC_ADDR16_HA,         "ADDR16_HA" },
+  { RELOC_POWERPC32::R_PPC_ADDR14,            "ADDR14" },
+  { RELOC_POWERPC32::R_PPC_ADDR14_BRTAKEN,    "ADDR14_BRTAKEN" },
+  { RELOC_POWERPC32::R_PPC_ADDR14_BRNTAKEN,   "ADDR14_BRNTAKEN" },
+  { RELOC_POWERPC32::R_PPC_REL24,             "REL24" },
+  { RELOC_POWERPC32::R_PPC_REL14,             "REL14" },
+  { RELOC_POWERPC32::R_PPC_REL14_BRTAKEN,     "REL14_BRTAKEN" },
+  { RELOC_POWERPC32::R_PPC_REL14_BRNTAKEN,    "REL14_BRNTAKEN" },
+  { RELOC_POWERPC32::R_PPC_GOT16,             "GOT16" },
+  { RELOC_POWERPC32::R_PPC_GOT16_LO,          "GOT16_LO" },
+  { RELOC_POWERPC32::R_PPC_GOT16_HI,          "GOT16_HI" },
+  { RELOC_POWERPC32::R_PPC_GOT16_HA,          "GOT16_HA" },
+  { RELOC_POWERPC32::R_PPC_PLTREL24,          "PLTREL24" },
+  { RELOC_POWERPC32::R_PPC_JMP_SLOT,          "JMP_SLOT" },
+  { RELOC_POWERPC32::R_PPC_RELATIVE,          "RELATIVE" },
+  { RELOC_POWERPC32::R_PPC_LOCAL24PC,         "LOCAL24PC" },
+  { RELOC_POWERPC32::R_PPC_REL32,             "REL32" },
+  { RELOC_POWERPC32::R_PPC_TLS,               "TLS" },
+  { RELOC_POWERPC32::R_PPC_DTPMOD32,          "DTPMOD32" },
+  { RELOC_POWERPC32::R_PPC_TPREL16,           "TPREL16" },
+  { RELOC_POWERPC32::R_PPC_TPREL16_LO,        "TPREL16_LO" },
+  { RELOC_POWERPC32::R_PPC_TPREL16_HI,        "TPREL16_HI" },
+  { RELOC_POWERPC32::R_PPC_TPREL16_HA,        "TPREL16_HA" },
+  { RELOC_POWERPC32::R_PPC_TPREL32,           "TPREL32" },
+  { RELOC_POWERPC32::R_PPC_DTPREL16,          "DTPREL16" },
+  { RELOC_POWERPC32::R_PPC_DTPREL16_LO,       "DTPREL16_LO" },
+  { RELOC_POWERPC32::R_PPC_DTPREL16_HI,       "DTPREL16_HI" },
+  { RELOC_POWERPC32::R_PPC_DTPREL16_HA,       "DTPREL16_HA" },
+  { RELOC_POWERPC32::R_PPC_DTPREL32,          "DTPREL32" },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSGD16,       "GOT_TLSGD16" },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSGD16_LO,    "GOT_TLSGD16_LO" },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSGD16_HI,    "GOT_TLSGD16_HI" },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSGD16_HA,    "GOT_TLSGD16_HA" },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSLD16,       "GOT_TLSLD16" },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSLD16_LO,    "GOT_TLSLD16_LO" },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSLD16_HI,    "GOT_TLSLD16_HI" },
+  { RELOC_POWERPC32::R_PPC_GOT_TLSLD16_HA,    "GOT_TLSLD16_HA" },
+  { RELOC_POWERPC32::R_PPC_GOT_TPREL16,       "GOT_TPREL16" },
+  { RELOC_POWERPC32::R_PPC_GOT_TPREL16_LO,    "GOT_TPREL16_LO" },
+  { RELOC_POWERPC32::R_PPC_GOT_TPREL16_HI,    "GOT_TPREL16_HI" },
+  { RELOC_POWERPC32::R_PPC_GOT_TPREL16_HA,    "GOT_TPREL16_HA" },
+  { RELOC_POWERPC32::R_PPC_GOT_DTPREL16,      "GOT_DTPREL16" },
+  { RELOC_POWERPC32::R_PPC_GOT_DTPREL16_LO,   "GOT_DTPREL16_LO" },
+  { RELOC_POWERPC32::R_PPC_GOT_DTPREL16_HI,   "GOT_DTPREL16_HI" },
+  { RELOC_POWERPC32::R_PPC_GOT_DTPREL16_HA,   "GOT_DTPREL16_HA" },
+  { RELOC_POWERPC32::R_PPC_TLSGD,             "TLSGD" },
+  { RELOC_POWERPC32::R_PPC_TLSLD,             "TLSLD" },
+  { RELOC_POWERPC32::R_PPC_REL16,             "REL16" },
+  { RELOC_POWERPC32::R_PPC_REL16_LO,          "REL16_LO" },
+  { RELOC_POWERPC32::R_PPC_REL16_HI,          "REL16_HI" },
+  { RELOC_POWERPC32::R_PPC_REL16_HA,          "REL16_HA" },
+  };
+  auto   it  = enumStrings.find(e);
+  return it == enumStrings.end() ? "UNDEFINED" : it->second;
+}
+
+const char* to_string(RELOC_POWERPC64 e) {
+  CONST_MAP(RELOC_POWERPC64, const char*, 84) enumStrings {
+  { RELOC_POWERPC64::R_PPC64_NONE,                "NONE" },
+  { RELOC_POWERPC64::R_PPC64_ADDR32,              "ADDR32" },
+  { RELOC_POWERPC64::R_PPC64_ADDR24,              "ADDR24" },
+  { RELOC_POWERPC64::R_PPC64_ADDR16,              "ADDR16" },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_LO,           "ADDR16_LO" },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HI,           "ADDR16_HI" },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HA,           "ADDR16_HA" },
+  { RELOC_POWERPC64::R_PPC64_ADDR14,              "ADDR14" },
+  { RELOC_POWERPC64::R_PPC64_ADDR14_BRTAKEN,      "ADDR14_BRTAKEN" }, 
+  { RELOC_POWERPC64::R_PPC64_ADDR14_BRNTAKEN,     "ADDR14_BRNTAKEN" },
+  { RELOC_POWERPC64::R_PPC64_REL24,               "REL24" },
+  { RELOC_POWERPC64::R_PPC64_REL14,               "REL14" },
+  { RELOC_POWERPC64::R_PPC64_REL14_BRTAKEN,       "REL14_BRTAKEN" },
+  { RELOC_POWERPC64::R_PPC64_REL14_BRNTAKEN,      "REL14_BRNTAKEN" },
+  { RELOC_POWERPC64::R_PPC64_GOT16,               "GOT16" },
+  { RELOC_POWERPC64::R_PPC64_GOT16_LO,            "GOT16_LO" },
+  { RELOC_POWERPC64::R_PPC64_GOT16_HI,            "GOT16_HI" },
+  { RELOC_POWERPC64::R_PPC64_GOT16_HA,            "GOT16_HA" },
+  { RELOC_POWERPC64::R_PPC64_JMP_SLOT,            "JMP_SLOT" },
+  { RELOC_POWERPC64::R_PPC64_RELATIVE,            "RELATIVE"},
+  { RELOC_POWERPC64::R_PPC64_REL32,               "REL32" },
+  { RELOC_POWERPC64::R_PPC64_ADDR64,              "ADDR64" },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HIGHER,       "ADDR16_HIGHER" },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HIGHERA,      "ADDR16_HIGHERA" },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HIGHEST,      "ADDR16_HIGHEST" },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_HIGHESTA,     "ADDR16_HIGHESTA" },
+  { RELOC_POWERPC64::R_PPC64_REL64,               "REL64" },
+  { RELOC_POWERPC64::R_PPC64_TOC16,               "TOC16" },
+  { RELOC_POWERPC64::R_PPC64_TOC16_LO,            "TOC16_LO" },
+  { RELOC_POWERPC64::R_PPC64_TOC16_HI,            "TOC16_HI" },
+  { RELOC_POWERPC64::R_PPC64_TOC16_HA,            "TOC16_HA" },
+  { RELOC_POWERPC64::R_PPC64_TOC,                 "TOC" },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_DS,           "ADDR16_DS" },
+  { RELOC_POWERPC64::R_PPC64_ADDR16_LO_DS,        "ADDR16_LO_DS" },
+  { RELOC_POWERPC64::R_PPC64_GOT16_DS,            "GOT16_DS" },
+  { RELOC_POWERPC64::R_PPC64_GOT16_LO_DS,         "GOT16_LO_DS" },
+  { RELOC_POWERPC64::R_PPC64_TOC16_DS,            "TOC16_DS" },
+  { RELOC_POWERPC64::R_PPC64_TOC16_LO_DS,         "TOC16_LO_DS" },
+  { RELOC_POWERPC64::R_PPC64_TLS,                 "TLS" },
+  { RELOC_POWERPC64::R_PPC64_DTPMOD64,            "DTPMOD64" },
+  { RELOC_POWERPC64::R_PPC64_TPREL16,             "TPREL16" },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_LO,          "TPREL16_LO" },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HI,          "TPREL16_HI" },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HA,          "TPREL16_HA" },
+  { RELOC_POWERPC64::R_PPC64_TPREL64,             "TPREL64" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16,            "DTPREL16" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_LO,         "DTPREL16_LO" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HI,         "DTPREL16_HI" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HA,         "DTPREL16_HA" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL64,            "DTPREL64" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSGD16,         "GOT_TLSGD16" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSGD16_LO,      "GOT_TLSGD16_LO" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSGD16_HI,      "GOT_TLSGD16_HI" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSGD16_HA,      "GOT_TLSGD16_HA" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSLD16,         "GOT_TLSLD16" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSLD16_LO,      "GOT_TLSLD16_LO" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSLD16_HI,      "GOT_TLSLD16_HI" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TLSLD16_HA,      "GOT_TLSLD16_HA" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TPREL16_DS,      "GOT_TPREL16_DS" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TPREL16_LO_DS,   "GOT_TPREL16_LO_DS" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TPREL16_HI,      "GOT_TPREL16_HI" },
+  { RELOC_POWERPC64::R_PPC64_GOT_TPREL16_HA,      "GOT_TPREL16_HA" },
+  { RELOC_POWERPC64::R_PPC64_GOT_DTPREL16_DS,     "GOT_DTPREL16_DS" },
+  { RELOC_POWERPC64::R_PPC64_GOT_DTPREL16_LO_DS,  "GOT_DTPREL16_LO_DS" },
+  { RELOC_POWERPC64::R_PPC64_GOT_DTPREL16_HI,     "GOT_DTPREL16_HI" },
+  { RELOC_POWERPC64::R_PPC64_GOT_DTPREL16_HA,     "GOT_DTPREL16_HA" },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_DS,          "TPREL16_DS" },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_LO_DS,       "TPREL16_LO_DS" },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HIGHER,      "TPREL16_HIGHER" },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HIGHERA,     "TPREL16_HIGHERA" },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HIGHEST,     "TPREL16_HIGHEST" },
+  { RELOC_POWERPC64::R_PPC64_TPREL16_HIGHESTA,    "TPREL16_HIGHESTA" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_DS,         "DTPREL16_DS" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_LO_DS,      "DTPREL16_LO_DS" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HIGHER,     "DTPREL16_HIGHER" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HIGHERA,    "DTPREL16_HIGHERA" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HIGHEST,    "DTPREL16_HIGHEST" },
+  { RELOC_POWERPC64::R_PPC64_DTPREL16_HIGHESTA,   "DTPREL16_HIGHESTA" },
+  { RELOC_POWERPC64::R_PPC64_TLSGD,               "TLSGD" },
+  { RELOC_POWERPC64::R_PPC64_TLSLD,               "TLSLD" },
+  { RELOC_POWERPC64::R_PPC64_REL16,               "REL16" },
+  { RELOC_POWERPC64::R_PPC64_REL16_LO,            "REL16_LO" },
+  { RELOC_POWERPC64::R_PPC64_REL16_HI,            "REL16_HI" },
+  { RELOC_POWERPC64::R_PPC64_REL16_HA,            "REL16_HA" },
+  };
+  auto   it  = enumStrings.find(e);
+  return it == enumStrings.end() ? "UNDEFINED" : it->second;
+}
+
 const char* to_string(ELF_CLASS e) {
-  const std::map<ELF_CLASS, const char*> enumStrings {
+  CONST_MAP(ELF_CLASS, const char*, 3) enumStrings {
     { ELF_CLASS::ELFCLASSNONE, "NONE"},
     { ELF_CLASS::ELFCLASS32,   "CLASS32"},
     { ELF_CLASS::ELFCLASS64,   "CLASS64"},
@@ -854,7 +1015,7 @@ const char* to_string(ELF_CLASS e) {
 }
 
 const char* to_string(ELF_DATA e) {
-  const std::map<ELF_DATA, const char*> enumStrings {
+  CONST_MAP(ELF_DATA, const char*, 3) enumStrings {
     { ELF_DATA::ELFDATANONE, "NONE"},
     { ELF_DATA::ELFDATA2LSB, "LSB"},
     { ELF_DATA::ELFDATA2MSB, "MSB"},
@@ -865,7 +1026,7 @@ const char* to_string(ELF_DATA e) {
 }
 
 const char* to_string(OS_ABI e) {
-  const std::map<OS_ABI, const char*> enumStrings {
+  CONST_MAP(OS_ABI, const char*, 23) enumStrings {
     { OS_ABI::ELFOSABI_SYSTEMV,      "SYSTEMV"},
     { OS_ABI::ELFOSABI_HPUX,         "HPUX"},
     { OS_ABI::ELFOSABI_NETBSD,       "NETBSD"},
@@ -897,7 +1058,7 @@ const char* to_string(OS_ABI e) {
 
 
 const char* to_string(DYNSYM_COUNT_METHODS e) {
-  const std::map<DYNSYM_COUNT_METHODS, const char*> enumStrings {
+  CONST_MAP(DYNSYM_COUNT_METHODS, const char*, 4) enumStrings {
     { DYNSYM_COUNT_METHODS::COUNT_AUTO,        "AUTO"},
     { DYNSYM_COUNT_METHODS::COUNT_SECTION,     "SECTION"},
     { DYNSYM_COUNT_METHODS::COUNT_HASH,        "HASH"},
@@ -910,7 +1071,8 @@ const char* to_string(DYNSYM_COUNT_METHODS e) {
 
 
 const char* to_string(NOTE_TYPES e) {
-  const std::map<NOTE_TYPES, const char*> enumStrings {
+  CONST_MAP(NOTE_TYPES, const char*, 5) enumStrings {
+    { NOTE_TYPES::NT_UNKNOWN,          "UNKNOWN"},
     { NOTE_TYPES::NT_GNU_ABI_TAG,      "ABI_TAG"},
     { NOTE_TYPES::NT_GNU_HWCAP,        "HWCAP"},
     { NOTE_TYPES::NT_GNU_BUILD_ID,     "BUILD_ID"},
@@ -922,7 +1084,8 @@ const char* to_string(NOTE_TYPES e) {
 }
 
 const char* to_string(NOTE_ABIS e) {
-  const std::map<NOTE_ABIS, const char*> enumStrings {
+  CONST_MAP(NOTE_ABIS, const char*, 7) enumStrings {
+    { NOTE_ABIS::ELF_NOTE_UNKNOWN,     "UNKNOWN"},
     { NOTE_ABIS::ELF_NOTE_OS_LINUX,    "LINUX"},
     { NOTE_ABIS::ELF_NOTE_OS_GNU,      "GNU"},
     { NOTE_ABIS::ELF_NOTE_OS_SOLARIS2, "SOLARIS2"},
@@ -936,7 +1099,7 @@ const char* to_string(NOTE_ABIS e) {
 }
 
 const char* to_string(RELOCATION_PURPOSES e) {
-  const std::map<RELOCATION_PURPOSES, const char*> enumStrings {
+  CONST_MAP(RELOCATION_PURPOSES, const char*, 4) enumStrings {
     { RELOCATION_PURPOSES::RELOC_PURPOSE_NONE,    "NONE"},
     { RELOCATION_PURPOSES::RELOC_PURPOSE_PLTGOT,  "PLTGOT"},
     { RELOCATION_PURPOSES::RELOC_PURPOSE_DYNAMIC, "DYNAMIC"},
@@ -949,7 +1112,7 @@ const char* to_string(RELOCATION_PURPOSES e) {
 
 
 const char* to_string(PPC64_EFLAGS e) {
-  const std::map<PPC64_EFLAGS, const char*> enumStrings {
+  CONST_MAP(PPC64_EFLAGS, const char*, 1) enumStrings {
     { PPC64_EFLAGS::EF_PPC64_ABI, "ABI"},
   };
 
@@ -959,7 +1122,7 @@ const char* to_string(PPC64_EFLAGS e) {
 
 
 const char* to_string(ARM_EFLAGS e) {
-  const std::map<ARM_EFLAGS, const char*> enumStrings {
+  CONST_MAP(ARM_EFLAGS, const char*, 8) enumStrings {
     { ARM_EFLAGS::EF_ARM_SOFT_FLOAT,   "SOFT_FLOAT" },
     { ARM_EFLAGS::EF_ARM_VFP_FLOAT,    "VFP_FLOAT"  },
     { ARM_EFLAGS::EF_ARM_EABI_UNKNOWN, "UNKNOWN"    },
@@ -975,7 +1138,7 @@ const char* to_string(ARM_EFLAGS e) {
 }
 
 const char* to_string(MIPS_EFLAGS e) {
-  const std::map<MIPS_EFLAGS, const char*> enumStrings {
+  CONST_MAP(MIPS_EFLAGS, const char*, 43) enumStrings {
     { MIPS_EFLAGS::EF_MIPS_NOREORDER,     "NOREORDER"     },
     { MIPS_EFLAGS::EF_MIPS_PIC,           "PIC"           },
     { MIPS_EFLAGS::EF_MIPS_CPIC,          "CPIC"          },
@@ -1031,7 +1194,7 @@ const char* to_string(MIPS_EFLAGS e) {
 
 
 const char* to_string(HEXAGON_EFLAGS e) {
-  const std::map<HEXAGON_EFLAGS, const char*> enumStrings {
+  CONST_MAP(HEXAGON_EFLAGS, const char*, 9) enumStrings {
     { HEXAGON_EFLAGS::EF_HEXAGON_MACH_V2,   "MACH_V2"  },
     { HEXAGON_EFLAGS::EF_HEXAGON_MACH_V3,   "MACH_V3"  },
     { HEXAGON_EFLAGS::EF_HEXAGON_MACH_V4,   "MACH_V4"  },
@@ -1050,7 +1213,7 @@ const char* to_string(HEXAGON_EFLAGS e) {
 }
 
 const char* to_string(IDENTITY e) {
-  const std::map<IDENTITY, const char*> enumStrings {
+  CONST_MAP(IDENTITY, const char*, 11) enumStrings {
     { IDENTITY::EI_MAG0,       "MAG0"       },
     { IDENTITY::EI_MAG1,       "MAG1"       },
     { IDENTITY::EI_MAG2,       "MAG2"       },
@@ -1069,7 +1232,7 @@ const char* to_string(IDENTITY e) {
 }
 
 const char* to_string(SYMBOL_SECTION_INDEX e) {
-  const std::map<SYMBOL_SECTION_INDEX, const char*> enumStrings {
+  CONST_MAP(SYMBOL_SECTION_INDEX, const char*, 10) enumStrings {
     { SYMBOL_SECTION_INDEX::SHN_UNDEF,     "UNDEF"     },
     { SYMBOL_SECTION_INDEX::SHN_LORESERVE, "LORESERVE" },
     { SYMBOL_SECTION_INDEX::SHN_LOPROC,    "LOPROC"    },
@@ -1088,7 +1251,7 @@ const char* to_string(SYMBOL_SECTION_INDEX e) {
 
 
 const char* to_string(DYNAMIC_FLAGS e) {
-  const std::map<DYNAMIC_FLAGS, const char*> enum_strings {
+  CONST_MAP(DYNAMIC_FLAGS, const char*, 5) enum_strings {
     { DYNAMIC_FLAGS::DF_ORIGIN,     "ORIGIN"         },
     { DYNAMIC_FLAGS::DF_SYMBOLIC,   "SYMBOLIC"       },
     { DYNAMIC_FLAGS::DF_TEXTREL,    "TEXTREL"        },
@@ -1101,7 +1264,7 @@ const char* to_string(DYNAMIC_FLAGS e) {
 }
 
 const char* to_string(DYNAMIC_FLAGS_1 e) {
-  const std::map<DYNAMIC_FLAGS_1, const char*> enum_strings_flags1 {
+  CONST_MAP(DYNAMIC_FLAGS_1, const char*, 26) enum_strings_flags1 {
     { DYNAMIC_FLAGS_1::DF_1_NOW,        "NOW"        },
     { DYNAMIC_FLAGS_1::DF_1_GLOBAL,     "GLOBAL"     },
     { DYNAMIC_FLAGS_1::DF_1_GROUP,      "GROUP"      },
@@ -1135,10 +1298,24 @@ const char* to_string(DYNAMIC_FLAGS_1 e) {
 }
 
 const char* to_string(ELF_SEGMENT_FLAGS e) {
-  const std::map<ELF_SEGMENT_FLAGS, const char*> enum_strings {
-    { ELF_SEGMENT_FLAGS::PF_X, "X" },
-    { ELF_SEGMENT_FLAGS::PF_W, "W" },
-    { ELF_SEGMENT_FLAGS::PF_R, "R" },
+  CONST_MAP(ELF_SEGMENT_FLAGS, const char*, 4) enum_strings {
+    { ELF_SEGMENT_FLAGS::PF_NONE, "NONE" },
+    { ELF_SEGMENT_FLAGS::PF_X,    "X" },
+    { ELF_SEGMENT_FLAGS::PF_W,    "W" },
+    { ELF_SEGMENT_FLAGS::PF_R,    "R" },
+  };
+
+  auto   it  = enum_strings.find(e);
+  return it == enum_strings.end() ? "UNDEFINED" : it->second;
+}
+
+
+const char* to_string(ELF_SYMBOL_VISIBILITY e) {
+  CONST_MAP(ELF_SYMBOL_VISIBILITY, const char*, 4) enum_strings {
+    { ELF_SYMBOL_VISIBILITY::STV_DEFAULT,   "DEFAULT"   },
+    { ELF_SYMBOL_VISIBILITY::STV_HIDDEN,    "HIDDEN"    },
+    { ELF_SYMBOL_VISIBILITY::STV_INTERNAL,  "INTERNAL"  },
+    { ELF_SYMBOL_VISIBILITY::STV_PROTECTED, "PROTECTED" },
   };
 
   auto   it  = enum_strings.find(e);
@@ -1148,6 +1325,7 @@ const char* to_string(ELF_SEGMENT_FLAGS e) {
 
 
 
+
 } // namespace ELF
 } // namespace LIEF
 
