@@ -14,16 +14,19 @@
  * limitations under the License.
  */
 
-#include "easylogging++.h"
+#include "LIEF/logging++.hpp"
 
 #include "LIEF/MachO/Binary.hpp"
 #include "LIEF/MachO/Builder.hpp"
 #include "Binary.tcc"
 
+#include "LIEF/MachO/hash.hpp"
+
 #include "LIEF/exception.hpp"
 
 #include <algorithm>
 #include <numeric>
+#include <sstream>
 
 #if defined(__unix__) || (defined(__APPLE__) && defined(__MACH__))
 #include <unistd.h>
@@ -52,7 +55,7 @@ LIEF::sections_t Binary::get_abstract_sections(void) {
 // LIEF Interface
 // ==============
 
-void Binary::patch_address(uint64_t address, const std::vector<uint8_t>& patch_value) {
+void Binary::patch_address(uint64_t address, const std::vector<uint8_t>& patch_value, LIEF::Binary::VA_TYPES) {
   // Find the segment associated with the virtual address
   SegmentCommand& segment_topatch = this->segment_from_virtual_address(address);
   const uint64_t offset = address - segment_topatch.virtual_address();
@@ -65,7 +68,7 @@ void Binary::patch_address(uint64_t address, const std::vector<uint8_t>& patch_v
 
 }
 
-void Binary::patch_address(uint64_t address, uint64_t patch_value, size_t size) {
+void Binary::patch_address(uint64_t address, uint64_t patch_value, size_t size, LIEF::Binary::VA_TYPES) {
   if (size > sizeof(patch_value)) {
     throw std::runtime_error("Invalid size (" + std::to_string(size) + ")");
   }
@@ -82,7 +85,7 @@ void Binary::patch_address(uint64_t address, uint64_t patch_value, size_t size)
 
 }
 
-std::vector<uint8_t> Binary::get_content_from_virtual_address(uint64_t virtual_address, uint64_t size) const {
+std::vector<uint8_t> Binary::get_content_from_virtual_address(uint64_t virtual_address, uint64_t size, LIEF::Binary::VA_TYPES) const {
   const SegmentCommand& segment = this->segment_from_virtual_address(virtual_address);
   const std::vector<uint8_t>& content = segment.content();
   const uint64_t offset = virtual_address - segment.virtual_address();
@@ -420,7 +423,7 @@ const Section& Binary::section_from_virtual_address(uint64_t address) const {
       sections.cend(),
       [&address] (const Section& section) {
         return ((section.virtual_address() <= address) and
-            address <= (section.virtual_address() + section.size()));
+            address < (section.virtual_address() + section.size()));
       });
 
   if (it_section == sections.cend()) {
@@ -441,7 +444,7 @@ const SegmentCommand& Binary::segment_from_virtual_address(uint64_t virtual_addr
       std::end(segments),
       [&virtual_address] (const SegmentCommand& segment) {
         return ((segment.virtual_address() <= virtual_address) and
-            virtual_address <= (segment.virtual_address() + segment.virtual_size()));
+            virtual_address < (segment.virtual_address() + segment.virtual_size()));
       });
 
   if (it_segment == segments.cend()) {
@@ -464,7 +467,7 @@ const SegmentCommand& Binary::segment_from_offset(uint64_t offset) const {
       std::end(segments),
       [&offset] (const SegmentCommand& segment) {
         return ((segment.file_offset() <= offset) and
-            offset <= (segment.file_offset() + segment.file_size()));
+            offset < (segment.file_offset() + segment.file_size()));
       });
 
   if (it_segment == segments.cend()) {
@@ -754,18 +757,142 @@ const ThreadCommand& Binary::thread_command(void) const {
   return this->command<ThreadCommand>();
 }
 
+// RPath command
+// +++++++++++++
+bool Binary::has_rpath(void) const {
+  return this->has_command<RPathCommand>();
+}
+
+RPathCommand& Binary::rpath(void) {
+  return this->command<RPathCommand>();
+}
+
+const RPathCommand& Binary::rpath(void) const {
+  return this->command<RPathCommand>();
+}
+
+// SymbolCommand command
+// +++++++++++++++++++++
+bool Binary::has_symbol_command(void) const {
+  return this->has_command<SymbolCommand>();
+}
+
+SymbolCommand& Binary::symbol_command(void) {
+  return this->command<SymbolCommand>();
+}
+
+const SymbolCommand& Binary::symbol_command(void) const {
+  return this->command<SymbolCommand>();
+}
+
+// DynamicSymbolCommand command
+// ++++++++++++++++++++++++++++
+bool Binary::has_dynamic_symbol_command(void) const {
+  return this->has_command<SymbolCommand>();
+}
+
+DynamicSymbolCommand& Binary::dynamic_symbol_command(void) {
+  return this->command<DynamicSymbolCommand>();
+}
+
+const DynamicSymbolCommand& Binary::dynamic_symbol_command(void) const {
+  return this->command<DynamicSymbolCommand>();
+}
+
+// CodeSignature command
+// +++++++++++++++++++++
+bool Binary::has_code_signature(void) const {
+  return this->has_command<CodeSignature>();
+}
+
+CodeSignature& Binary::code_signature(void) {
+  return this->command<CodeSignature>();
+}
+
+const CodeSignature& Binary::code_signature(void) const {
+  return this->command<CodeSignature>();
+}
+
+
+// DataInCode command
+// ++++++++++++++++++
+bool Binary::has_data_in_code(void) const {
+  return this->has_command<DataInCode>();
+}
+
+DataInCode& Binary::data_in_code(void) {
+  return this->command<DataInCode>();
+}
+
+const DataInCode& Binary::data_in_code(void) const {
+  return this->command<DataInCode>();
+}
+
+
+// SegmentSplitInfo command
+// ++++++++++++++++++++++++
+bool Binary::has_segment_split_info(void) const {
+  return this->has_command<SegmentSplitInfo>();
+}
+
+SegmentSplitInfo& Binary::segment_split_info(void) {
+  return this->command<SegmentSplitInfo>();
+}
+
+const SegmentSplitInfo& Binary::segment_split_info(void) const {
+  return this->command<SegmentSplitInfo>();
+}
+
+
+// SubFramework command
+// ++++++++++++++++++++
+bool Binary::has_sub_framework(void) const {
+  return this->has_command<SubFramework>();
+}
+
+SubFramework& Binary::sub_framework(void) {
+  return this->command<SubFramework>();
+}
+
+const SubFramework& Binary::sub_framework(void) const {
+  return this->command<SubFramework>();
+}
+
+// DyldEnvironment command
+// +++++++++++++++++++++++
+bool Binary::has_dyld_environment(void) const {
+  return this->has_command<DyldEnvironment>();
+}
+
+DyldEnvironment& Binary::dyld_environment(void) {
+  return this->command<DyldEnvironment>();
+}
+
+const DyldEnvironment& Binary::dyld_environment(void) const {
+  return this->command<DyldEnvironment>();
+}
+
+// EncryptionInfo command
+// +++++++++++++++++++++++
+bool Binary::has_encryption_info(void) const {
+  return this->has_command<EncryptionInfo>();
+}
+
+EncryptionInfo& Binary::encryption_info(void) {
+  return this->command<EncryptionInfo>();
+}
+
+const EncryptionInfo& Binary::encryption_info(void) const {
+  return this->command<EncryptionInfo>();
+}
+
+
+
 
 
 
 void Binary::accept(LIEF::Visitor& visitor) const {
-  visitor(this->header());
-  for (const LoadCommand& cmd : this->commands()) {
-    visitor(cmd);
-  }
-
-  for (const Symbol& symbol : this->symbols()) {
-    visitor(symbol);
-  }
+  visitor.visit(*this);
 }
 
 
