@@ -20,9 +20,9 @@
 #include <numeric>
 #include <limits>
 
-#include "easylogging++.h"
+#include "LIEF/logging++.hpp"
 
-#include "LIEF/visitors/Hash.hpp"
+#include "LIEF/PE/hash.hpp"
 #include "LIEF/exception.hpp"
 #include "LIEF/utils.hpp"
 
@@ -81,6 +81,7 @@ Binary::Binary(void) :
   rich_header_{},
   header_{},
   optional_header_{},
+  available_sections_space_{0},
   has_rich_header_{false},
   has_tls_{false},
   has_imports_{false},
@@ -302,7 +303,7 @@ DataDirectory& Binary::data_directory(DATA_DIRECTORY index) {
 
 const DataDirectory& Binary::data_directory(DATA_DIRECTORY index) const {
   if (static_cast<size_t>(index) < this->data_directories_.size() and this->data_directories_[static_cast<size_t>(index)] != nullptr) {
-    return *this->data_directories_[index];
+    return *this->data_directories_[static_cast<size_t>(index)];
   } else {
     throw not_found("Data directory doesn't exist");
   }
@@ -516,8 +517,26 @@ void Binary::delete_section(const std::string& name) {
       std::end(this->sections_));
 }
 
+void Binary::make_space_for_new_section(void) {
+  const uint32_t shift = align(sizeof(pe_section), this->optional_header().file_alignment());
+  VLOG(VDEBUG) << "Making space for a new section header";
+  VLOG(VDEBUG) << "Shifting all sections by " << std::hex << std::showbase << shift;
+
+  // Shift offset of the section content by the size of
+  // a section header aligned on "file alignment"
+  for (Section* section : this->sections_) {
+    section->pointerto_raw_data(section->pointerto_raw_data() + shift);
+  }
+  this->available_sections_space_++;
+}
+
 Section& Binary::add_section(const Section& section, PE_SECTION_TYPES type) {
 
+  if (this->available_sections_space_ < 0) {
+    this->make_space_for_new_section();
+    return this->add_section(section, type);
+  }
+
   // Check if a section of type **type** already exist
   auto&& it_section = std::find_if(
       std::begin(this->sections_),
@@ -634,6 +653,8 @@ Section& Binary::add_section(const Section& section, PE_SECTION_TYPES type) {
   if (this->sections_.size() >= std::numeric_limits<uint16_t>::max()) {
     throw pe_error("Binary reachs its maximum number of sections");
   }
+
+  this->available_sections_space_--;
   this->sections_.push_back(new_section);
 
   // Update headers
@@ -641,7 +662,6 @@ Section& Binary::add_section(const Section& section, PE_SECTION_TYPES type) {
 
   this->optional_header().sizeof_image(this->virtual_size());
   this->optional_header().sizeof_headers(this->sizeof_headers());
-
   return *(this->sections_.back());
 }
 
@@ -747,7 +767,8 @@ uint32_t Binary::predict_function_rva(const std::string& library, const std::str
       });
 
   if (it_import == std::end(this->imports_)) {
-    throw not_found("Unable to find library '" + library + "'");
+    LOG(ERROR) << "Unable to find library '" << library << "'";
+    return 0;
   }
 
   it_const_import_entries entries = it_import->entries();
@@ -762,11 +783,13 @@ uint32_t Binary::predict_function_rva(const std::string& library, const std::str
       });
 
   if (nb_functions == 0) {
-    throw not_found("Unable to find the function '" + function + "' in '" + library + "'.");
+    LOG(ERROR) << "Unable to find the function '" << function << "' in '" << library + "'.";
+    return 0;
   }
 
   if (nb_functions > 1) {
-    throw not_supported("'" + function + "' is defined " + std::to_string(nb_functions) + " in '" + library + "'.");
+    LOG(ERROR) << "'" << function << "' is defined " << std::to_string(nb_functions) << " in '" << library << "'.";
+    return 0;
   }
 
   uint32_t import_table_size = static_cast<uint32_t>((this->imports().size() + 1) * sizeof(pe_import)); // +1 for the null entry
@@ -858,11 +881,7 @@ Export& Binary::get_export(void) {
 
 
 const Export& Binary::get_export(void) const {
-  if (this->has_exports()) {
-    return this->export_;
-  } else {
-    throw not_found("The binary doesn't have exports");
-  }
+  return this->export_;
 }
 
 /////////////////////////////////////
@@ -872,14 +891,12 @@ const Export& Binary::get_export(void) const {
 /////////////////////////////////////
 
 void Binary::set_resources(const ResourceDirectory& resource) {
-  // TODO: DELETE !!!!!!!!!
   delete this->resources_;
   this->resources_ = new ResourceDirectory{resource};
 }
 
 
 void Binary::set_resources(const ResourceData& resource) {
-  // TODO: DELETE !!!!!!!!!
   delete this->resources_;
   this->resources_ = new ResourceData{resource};
 }
@@ -927,9 +944,6 @@ const Debug& Binary::debug(void) const {
 /////////////////////
 
 const Signature& Binary::signature(void) const {
-  if (not this->has_signature()) {
-    throw not_found("Signature not found");
-  }
   return this->signature_;
 }
 
@@ -1025,7 +1039,7 @@ void Binary::hook_function(const std::string& function, uint64_t address) {
     }
   }
 
-  throw not_found("Unable to find library associated with function '" + function + "'");
+  LOG(WARNING) << "Unable to find library associated with function '" << function << "'";
 }
 
 
@@ -1039,11 +1053,21 @@ uint64_t Binary::entrypoint(void) const {
   return this->optional_header().imagebase() + this->optional_header().addressof_entrypoint();
 }
 
-void Binary::patch_address(uint64_t address, const std::vector<uint8_t>& patch_value) {
+void Binary::patch_address(uint64_t address, const std::vector<uint8_t>& patch_value, LIEF::Binary::VA_TYPES addr_type) {
+  uint64_t rva = address;
+
+  if (addr_type == LIEF::Binary::VA_TYPES::VA or addr_type == LIEF::Binary::VA_TYPES::AUTO) {
+    const int64_t delta = address - this->optional_header().imagebase();
+
+    if (delta > 0 or addr_type == LIEF::Binary::VA_TYPES::VA) {
+      rva -= this->optional_header().imagebase();
+    }
+  }
+
   // Find the section associated with the virtual address
-  Section& section_topatch = this->section_from_rva(address);
-  const uint64_t offset = address - section_topatch.virtual_address();
-  std::vector<uint8_t>& content = section_topatch.content();
+  Section& section_topatch = this->section_from_rva(rva);
+  const uint64_t offset = rva - section_topatch.virtual_address();
+  std::vector<uint8_t>& content = section_topatch.content_ref();
   std::copy(
       std::begin(patch_value),
       std::end(patch_value),
@@ -1051,14 +1075,25 @@ void Binary::patch_address(uint64_t address, const std::vector<uint8_t>& patch_v
 
 }
 
-void Binary::patch_address(uint64_t address, uint64_t patch_value, size_t size) {
+void Binary::patch_address(uint64_t address, uint64_t patch_value, size_t size, LIEF::Binary::VA_TYPES addr_type) {
   if (size > sizeof(patch_value)) {
-    throw std::runtime_error("Invalid size (" + std::to_string(size) + ")");
+    LOG(ERROR) << "Invalid size (" << std::to_string(size) << ")";
+    return;
   }
 
-  Section& section_topatch = this->section_from_rva(address);
-  const uint64_t offset = address - section_topatch.virtual_address();
-  std::vector<uint8_t>& content = section_topatch.content();
+  uint64_t rva = address;
+
+  if (addr_type == LIEF::Binary::VA_TYPES::VA or addr_type == LIEF::Binary::VA_TYPES::AUTO) {
+    const int64_t delta = address - this->optional_header().imagebase();
+
+    if (delta > 0 or addr_type == LIEF::Binary::VA_TYPES::VA) {
+      rva -= this->optional_header().imagebase();
+    }
+  }
+
+  Section& section_topatch = this->section_from_rva(rva);
+  const uint64_t offset = rva - section_topatch.virtual_address();
+  std::vector<uint8_t>& content = section_topatch.content_ref();
 
   std::copy(
       reinterpret_cast<uint8_t*>(&patch_value),
@@ -1067,14 +1102,18 @@ void Binary::patch_address(uint64_t address, uint64_t patch_value, size_t size)
 
 }
 
-std::vector<uint8_t> Binary::get_content_from_virtual_address(uint64_t virtual_address, uint64_t size) const {
+std::vector<uint8_t> Binary::get_content_from_virtual_address(uint64_t virtual_address, uint64_t size, LIEF::Binary::VA_TYPES addr_type) const {
   uint64_t rva = virtual_address;
-  const int64_t delta = virtual_address - this->optional_header().imagebase();
-  if (delta > 0) {
-    rva -= this->optional_header().imagebase();
+
+  if (addr_type == LIEF::Binary::VA_TYPES::VA or addr_type == LIEF::Binary::VA_TYPES::AUTO) {
+    const int64_t delta = virtual_address - this->optional_header().imagebase();
+
+    if (delta > 0 or addr_type == LIEF::Binary::VA_TYPES::VA) {
+      rva -= this->optional_header().imagebase();
+    }
   }
   const Section& section = this->section_from_rva(rva);
-  const std::vector<uint8_t>& content = section.content();
+  std::vector<uint8_t> content = section.content();
   const uint64_t offset = rva - section.virtual_address();
   uint64_t checked_size = size;
   if ((offset + checked_size) > content.size()) {
@@ -1127,9 +1166,6 @@ RichHeader& Binary::rich_header(void) {
 }
 
 const RichHeader& Binary::rich_header(void) const {
-  if (not this->has_rich_header()) {
-    throw not_found("Rich Header not found");
-  }
   return this->rich_header_;
 }
 
@@ -1157,51 +1193,7 @@ const ResourcesManager Binary::resources_manager(void) const {
 
 
 void Binary::accept(Visitor& visitor) const {
-  visitor(this->dos_header());
-  visitor(this->header());
-  visitor(this->optional_header());
-
-  for (const DataDirectory& data_directory : this->data_directories()) {
-    visitor(data_directory);
-  }
-
-  for (const Section& section : this->sections()) {
-    visitor(section);
-  }
-
-  if (this->has_imports()) {
-    for (const Import& import : this->imports()) {
-      visitor(import);
-    }
-  }
-
-  if (this->has_relocations()) {
-    for (const Relocation& relocation : this->relocations()) {
-      visitor(relocation);
-    }
-  }
-
-  if (this->has_debug()) {
-    visitor(this->debug());
-  }
-
-
-  if (this->has_exports()) {
-    visitor(this->get_export());
-  }
-
-  for (const Symbol& symbol : this->symbols()) {
-    visitor(symbol);
-  }
-
-  if (this->has_tls()) {
-    visitor(this->tls());
-  }
-
-  if (this->has_rich_header()) {
-    visitor(this->rich_header());
-  }
-
+  visitor.visit(*this);
 }
 
 
